package cn.ibizlab.assetmanagement.core.apply.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import cn.ibizlab.assetmanagement.core.apply.domain.EAMApply;
import cn.ibizlab.assetmanagement.core.apply.filter.EAMApplySearchContext;
import cn.ibizlab.assetmanagement.core.apply.service.IEAMApplyService;

import cn.ibizlab.assetmanagement.util.helper.CachedBeanCopier;
import cn.ibizlab.assetmanagement.util.helper.DEFieldCacheMap;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cn.ibizlab.assetmanagement.core.apply.mapper.EAMApplyMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;
import org.springframework.util.StringUtils;

/**
 * 实体[维护申请] 服务对象接口实现
 */
@Slf4j
@Service("EAMApplyServiceImpl")
public class EAMApplyServiceImpl extends ServiceImpl<EAMApplyMapper, EAMApply> implements IEAMApplyService {

    @Autowired
    @Lazy
    protected cn.ibizlab.assetmanagement.core.apply.service.IEAMApplyTypeService eamapplytypeService;
    @Autowired
    @Lazy
    protected cn.ibizlab.assetmanagement.core.asset.service.IEAMAssetService eamassetService;
    @Autowired
    @Lazy
    protected cn.ibizlab.assetmanagement.core.location.service.IEAMLocationService eamlocationService;
    @Autowired
    @Lazy
    protected cn.ibizlab.assetmanagement.core.workorder.service.IEAMServiceLevelService eamservicelevelService;

    protected int batchSize = 500;

    @Override
    @Transactional
    public boolean create(EAMApply et) {
        fillParentData(et);
        createIndexMajorEntityData(et);
        if(!this.retBool(this.baseMapper.insert(et)))
            return false;
        CachedBeanCopier.copy(get(et.getEamapplyid()),et);
        return true;
    }

    @Override
    public void createBatch(List<EAMApply> list) {
        list.forEach(item->fillParentData(item));
        this.saveBatch(list,batchSize);
    }

    @Override
    @Transactional
    public boolean update(EAMApply et) {
        fillParentData(et);
        wooriginService.update(eamapplyInheritMapping.toWoorigin(et));
        if(!update(et,(Wrapper) et.getUpdateWrapper(true).eq("eamapplyid",et.getEamapplyid())))
            return false;
        CachedBeanCopier.copy(get(et.getEamapplyid()),et);
        return true;
    }

    @Override
    public void updateBatch(List<EAMApply> list) {
        list.forEach(item->fillParentData(item));
        updateBatchById(list,batchSize);
    }

    @Override
    @Transactional
    public boolean remove(String key) {
        wooriginService.remove(key);
        boolean result=removeById(key);
        return result ;
    }

    @Override
    public void removeBatch(Collection<String> idList) {
        removeByIds(idList);
    }

    @Override
    @Transactional
    public EAMApply get(String key) {
        EAMApply et = getById(key);
        if(et==null){
            et=new EAMApply();
            et.setEamapplyid(key);
        }
        else{
        }
        return et;
    }

    @Override
    public EAMApply getDraft(EAMApply et) {
        fillParentData(et);
        return et;
    }

    @Override
    public boolean checkKey(EAMApply et) {
        return (!ObjectUtils.isEmpty(et.getEamapplyid()))&&(!Objects.isNull(this.getById(et.getEamapplyid())));
    }
    @Override
    @Transactional
    public boolean save(EAMApply et) {
        if(!saveOrUpdate(et))
            return false;
        return true;
    }

    @Override
    @Transactional
    public boolean saveOrUpdate(EAMApply et) {
        if (null == et) {
            return false;
        } else {
            return checkKey(et) ? this.update(et) : this.create(et);
        }
    }

    @Override
    public boolean saveBatch(Collection<EAMApply> list) {
        list.forEach(item->fillParentData(item));
        saveOrUpdateBatch(list,batchSize);
        return true;
    }

    @Override
    public void saveBatch(List<EAMApply> list) {
        list.forEach(item->fillParentData(item));
        saveOrUpdateBatch(list,batchSize);
    }


	@Override
    public List<EAMApply> selectByEamapplytypeid(String eamapplytypeid) {
        return baseMapper.selectByEamapplytypeid(eamapplytypeid);
    }

    @Override
    public void removeByEamapplytypeid(String eamapplytypeid) {
        this.remove(new QueryWrapper<EAMApply>().eq("eamapplytypeid",eamapplytypeid));
    }

	@Override
    public List<EAMApply> selectByAssetid(String assetid) {
        return baseMapper.selectByAssetid(assetid);
    }

    @Override
    public void removeByAssetid(String assetid) {
        this.remove(new QueryWrapper<EAMApply>().eq("assetid",assetid));
    }

	@Override
    public List<EAMApply> selectByEamlocationid(String eamlocationid) {
        return baseMapper.selectByEamlocationid(eamlocationid);
    }

    @Override
    public void removeByEamlocationid(String eamlocationid) {
        this.remove(new QueryWrapper<EAMApply>().eq("eamlocationid",eamlocationid));
    }

	@Override
    public List<EAMApply> selectByEamservicelevelid(String eamservicelevelid) {
        return baseMapper.selectByEamservicelevelid(eamservicelevelid);
    }

    @Override
    public void removeByEamservicelevelid(String eamservicelevelid) {
        this.remove(new QueryWrapper<EAMApply>().eq("eamservicelevelid",eamservicelevelid));
    }


    /**
     * 查询集合 DEFAULT
     */
    @Override
    public Page<EAMApply> searchDefault(EAMApplySearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<EAMApply> pages=baseMapper.searchDefault(context.getPages(),context,context.getSelectCond());
        return new PageImpl<EAMApply>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }



    /**
     * 为当前实体填充父数据（外键值文本、外键值附加数据）
     * @param et
     */
    private void fillParentData(EAMApply et){
        //实体关系[DER1N_EAMAPPLY_EAMAPPLYTYPE_EAMAPPLYTYPEID]
        if(!ObjectUtils.isEmpty(et.getEamapplytypeid())){
            cn.ibizlab.assetmanagement.core.apply.domain.EAMApplyType eamapplytype=et.getEamapplytype();
            if(ObjectUtils.isEmpty(eamapplytype)){
                cn.ibizlab.assetmanagement.core.apply.domain.EAMApplyType majorEntity=eamapplytypeService.get(et.getEamapplytypeid());
                et.setEamapplytype(majorEntity);
                eamapplytype=majorEntity;
            }
            et.setEamapplytypename(eamapplytype.getEamapplytypename());
        }
        //实体关系[DER1N_EAMAPPLY_EAMASSET_ASSETID]
        if(!ObjectUtils.isEmpty(et.getAssetid())){
            cn.ibizlab.assetmanagement.core.asset.domain.EAMAsset asset=et.getAsset();
            if(ObjectUtils.isEmpty(asset)){
                cn.ibizlab.assetmanagement.core.asset.domain.EAMAsset majorEntity=eamassetService.get(et.getAssetid());
                et.setAsset(majorEntity);
                asset=majorEntity;
            }
            et.setAssetname(asset.getAssetname());
        }
        //实体关系[DER1N_EAMAPPLY_EAMLOCATION_EAMLOCATIONID]
        if(!ObjectUtils.isEmpty(et.getEamlocationid())){
            cn.ibizlab.assetmanagement.core.location.domain.EAMLocation eamlocation=et.getEamlocation();
            if(ObjectUtils.isEmpty(eamlocation)){
                cn.ibizlab.assetmanagement.core.location.domain.EAMLocation majorEntity=eamlocationService.get(et.getEamlocationid());
                et.setEamlocation(majorEntity);
                eamlocation=majorEntity;
            }
            et.setEamlocationname(eamlocation.getEamlocationname());
        }
        //实体关系[DER1N_EAMAPPLY_EAMSERVICELEVEL_EAMSERVICELEVELID]
        if(!ObjectUtils.isEmpty(et.getEamservicelevelid())){
            cn.ibizlab.assetmanagement.core.workorder.domain.EAMServiceLevel eamservicelevel=et.getEamservicelevel();
            if(ObjectUtils.isEmpty(eamservicelevel)){
                cn.ibizlab.assetmanagement.core.workorder.domain.EAMServiceLevel majorEntity=eamservicelevelService.get(et.getEamservicelevelid());
                et.setEamservicelevel(majorEntity);
                eamservicelevel=majorEntity;
            }
            et.setEamservicelevelname(eamservicelevel.getEamservicelevelname());
        }
    }



    @Autowired
    cn.ibizlab.assetmanagement.core.apply.mapping.EAMApplyInheritMapping eamapplyInheritMapping;
    @Autowired
    @Lazy
    protected cn.ibizlab.assetmanagement.core.workorder.service.IWOOriginService wooriginService;

    /**
     * 创建索引主实体数据
     * @param et
     */
    private void createIndexMajorEntityData(EAMApply et){
        if(ObjectUtils.isEmpty(et.getEamapplyid()))
            et.setEamapplyid((String)et.getDefaultKey(true));
        cn.ibizlab.assetmanagement.core.workorder.domain.WOOrigin woorigin =eamapplyInheritMapping.toWoorigin(et);
        woorigin.set("woorigintype","APPLY");
        wooriginService.create(woorigin);
    }

    @Override
    public List<JSONObject> select(String sql, Map param){
        return this.baseMapper.selectBySQL(sql,param);
    }

    @Override
    @Transactional
    public boolean execute(String sql , Map param){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql,param);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }

    @Override
    public List<EAMApply> getEamapplyByIds(List<String> ids) {
         return this.listByIds(ids);
    }

    @Override
    public List<EAMApply> getEamapplyByEntities(List<EAMApply> entities) {
        List ids =new ArrayList();
        for(EAMApply entity : entities){
            Serializable id=entity.getEamapplyid();
            if(!ObjectUtils.isEmpty(id)){
                ids.add(id);
            }
        }
        if(ids.size()>0)
           return this.listByIds(ids);
        else
           return entities;
    }

}



