package cn.ibizlab.assetmanagement.core.apply.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import cn.ibizlab.assetmanagement.core.apply.domain.EAMApplyStateModelLine;
import cn.ibizlab.assetmanagement.core.apply.filter.EAMApplyStateModelLineSearchContext;
import cn.ibizlab.assetmanagement.core.apply.service.IEAMApplyStateModelLineService;

import cn.ibizlab.assetmanagement.util.helper.CachedBeanCopier;
import cn.ibizlab.assetmanagement.util.helper.DEFieldCacheMap;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cn.ibizlab.assetmanagement.core.apply.mapper.EAMApplyStateModelLineMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;
import org.springframework.util.StringUtils;

/**
 * 实体[维护申请状态模型明细] 服务对象接口实现
 */
@Slf4j
@Service("EAMApplyStateModelLineServiceImpl")
public class EAMApplyStateModelLineServiceImpl extends ServiceImpl<EAMApplyStateModelLineMapper, EAMApplyStateModelLine> implements IEAMApplyStateModelLineService {

    @Autowired
    @Lazy
    protected cn.ibizlab.assetmanagement.core.apply.service.IEAMApplyStateModelService eamapplystatemodelService;
    @Autowired
    @Lazy
    protected cn.ibizlab.assetmanagement.core.apply.service.IEAMApplyStateService eamapplystateService;

    protected int batchSize = 500;

    @Override
    @Transactional
    public boolean create(EAMApplyStateModelLine et) {
        if(!this.retBool(this.baseMapper.insert(et)))
            return false;
        CachedBeanCopier.copy(get(et.getEamapplystatemodellineid()),et);
        return true;
    }

    @Override
    public void createBatch(List<EAMApplyStateModelLine> list) {
        this.saveBatch(list,batchSize);
    }

    @Override
    @Transactional
    public boolean update(EAMApplyStateModelLine et) {
        if(!update(et,(Wrapper) et.getUpdateWrapper(true).eq("eamapplystatemodellineid",et.getEamapplystatemodellineid())))
            return false;
        CachedBeanCopier.copy(get(et.getEamapplystatemodellineid()),et);
        return true;
    }

    @Override
    public void updateBatch(List<EAMApplyStateModelLine> list) {
        updateBatchById(list,batchSize);
    }

    @Override
    @Transactional
    public boolean remove(String key) {
        boolean result=removeById(key);
        return result ;
    }

    @Override
    public void removeBatch(Collection<String> idList) {
        removeByIds(idList);
    }

    @Override
    @Transactional
    public EAMApplyStateModelLine get(String key) {
        EAMApplyStateModelLine et = getById(key);
        if(et==null){
            et=new EAMApplyStateModelLine();
            et.setEamapplystatemodellineid(key);
        }
        else{
        }
        return et;
    }

    @Override
    public EAMApplyStateModelLine getDraft(EAMApplyStateModelLine et) {
        return et;
    }

    @Override
    public boolean checkKey(EAMApplyStateModelLine et) {
        return (!ObjectUtils.isEmpty(et.getEamapplystatemodellineid()))&&(!Objects.isNull(this.getById(et.getEamapplystatemodellineid())));
    }
    @Override
    @Transactional
    public boolean save(EAMApplyStateModelLine et) {
        if(!saveOrUpdate(et))
            return false;
        return true;
    }

    @Override
    @Transactional
    public boolean saveOrUpdate(EAMApplyStateModelLine et) {
        if (null == et) {
            return false;
        } else {
            return checkKey(et) ? this.update(et) : this.create(et);
        }
    }

    @Override
    public boolean saveBatch(Collection<EAMApplyStateModelLine> list) {
        saveOrUpdateBatch(list,batchSize);
        return true;
    }

    @Override
    public void saveBatch(List<EAMApplyStateModelLine> list) {
        saveOrUpdateBatch(list,batchSize);
    }


	@Override
    public List<EAMApplyStateModelLine> selectByEamapplystatemodelid(String eamapplystatemodelid) {
        return baseMapper.selectByEamapplystatemodelid(eamapplystatemodelid);
    }

    @Override
    public void removeByEamapplystatemodelid(String eamapplystatemodelid) {
        this.remove(new QueryWrapper<EAMApplyStateModelLine>().eq("eamapplystatemodelid",eamapplystatemodelid));
    }

	@Override
    public List<EAMApplyStateModelLine> selectByEamapplystateid(String eamapplystateid) {
        return baseMapper.selectByEamapplystateid(eamapplystateid);
    }

    @Override
    public void removeByEamapplystateid(String eamapplystateid) {
        this.remove(new QueryWrapper<EAMApplyStateModelLine>().eq("eamapplystateid",eamapplystateid));
    }


    /**
     * 查询集合 DEFAULT
     */
    @Override
    public Page<EAMApplyStateModelLine> searchDefault(EAMApplyStateModelLineSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<EAMApplyStateModelLine> pages=baseMapper.searchDefault(context.getPages(),context,context.getSelectCond());
        return new PageImpl<EAMApplyStateModelLine>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }







    @Override
    public List<JSONObject> select(String sql, Map param){
        return this.baseMapper.selectBySQL(sql,param);
    }

    @Override
    @Transactional
    public boolean execute(String sql , Map param){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql,param);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }

    @Override
    public List<EAMApplyStateModelLine> getEamapplystatemodellineByIds(List<String> ids) {
         return this.listByIds(ids);
    }

    @Override
    public List<EAMApplyStateModelLine> getEamapplystatemodellineByEntities(List<EAMApplyStateModelLine> entities) {
        List ids =new ArrayList();
        for(EAMApplyStateModelLine entity : entities){
            Serializable id=entity.getEamapplystatemodellineid();
            if(!ObjectUtils.isEmpty(id)){
                ids.add(id);
            }
        }
        if(ids.size()>0)
           return this.listByIds(ids);
        else
           return entities;
    }

}



