package cn.ibizlab.assetmanagement.core.apply.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import cn.ibizlab.assetmanagement.core.apply.domain.EAMApplyState;
import cn.ibizlab.assetmanagement.core.apply.filter.EAMApplyStateSearchContext;
import cn.ibizlab.assetmanagement.core.apply.service.IEAMApplyStateService;

import cn.ibizlab.assetmanagement.util.helper.CachedBeanCopier;
import cn.ibizlab.assetmanagement.util.helper.DEFieldCacheMap;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cn.ibizlab.assetmanagement.core.apply.mapper.EAMApplyStateMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;
import org.springframework.util.StringUtils;

/**
 * 实体[维护申请状态] 服务对象接口实现
 */
@Slf4j
@Service("EAMApplyStateServiceImpl")
public class EAMApplyStateServiceImpl extends ServiceImpl<EAMApplyStateMapper, EAMApplyState> implements IEAMApplyStateService {

    @Autowired
    @Lazy
    protected cn.ibizlab.assetmanagement.core.apply.service.IEAMApplyStateModelLineService eamapplystatemodellineService;

    protected int batchSize = 500;

    @Override
    @Transactional
    public boolean create(EAMApplyState et) {
        if(!this.retBool(this.baseMapper.insert(et)))
            return false;
        CachedBeanCopier.copy(get(et.getEamapplystateid()),et);
        return true;
    }

    @Override
    public void createBatch(List<EAMApplyState> list) {
        this.saveBatch(list,batchSize);
    }

    @Override
    @Transactional
    public boolean update(EAMApplyState et) {
        if(!update(et,(Wrapper) et.getUpdateWrapper(true).eq("eamapplystateid",et.getEamapplystateid())))
            return false;
        CachedBeanCopier.copy(get(et.getEamapplystateid()),et);
        return true;
    }

    @Override
    public void updateBatch(List<EAMApplyState> list) {
        updateBatchById(list,batchSize);
    }

    @Override
    @Transactional
    public boolean remove(String key) {
        boolean result=removeById(key);
        return result ;
    }

    @Override
    public void removeBatch(Collection<String> idList) {
        removeByIds(idList);
    }

    @Override
    @Transactional
    public EAMApplyState get(String key) {
        EAMApplyState et = getById(key);
        if(et==null){
            et=new EAMApplyState();
            et.setEamapplystateid(key);
        }
        else{
        }
        return et;
    }

    @Override
    public EAMApplyState getDraft(EAMApplyState et) {
        return et;
    }

    @Override
    public boolean checkKey(EAMApplyState et) {
        return (!ObjectUtils.isEmpty(et.getEamapplystateid()))&&(!Objects.isNull(this.getById(et.getEamapplystateid())));
    }
    @Override
    @Transactional
    public boolean save(EAMApplyState et) {
        if(!saveOrUpdate(et))
            return false;
        return true;
    }

    @Override
    @Transactional
    public boolean saveOrUpdate(EAMApplyState et) {
        if (null == et) {
            return false;
        } else {
            return checkKey(et) ? this.update(et) : this.create(et);
        }
    }

    @Override
    public boolean saveBatch(Collection<EAMApplyState> list) {
        saveOrUpdateBatch(list,batchSize);
        return true;
    }

    @Override
    public void saveBatch(List<EAMApplyState> list) {
        saveOrUpdateBatch(list,batchSize);
    }



    /**
     * 查询集合 DEFAULT
     */
    @Override
    public Page<EAMApplyState> searchDefault(EAMApplyStateSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<EAMApplyState> pages=baseMapper.searchDefault(context.getPages(),context,context.getSelectCond());
        return new PageImpl<EAMApplyState>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }







    @Override
    public List<JSONObject> select(String sql, Map param){
        return this.baseMapper.selectBySQL(sql,param);
    }

    @Override
    @Transactional
    public boolean execute(String sql , Map param){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql,param);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }

    @Override
    public List<EAMApplyState> getEamapplystateByIds(List<String> ids) {
         return this.listByIds(ids);
    }

    @Override
    public List<EAMApplyState> getEamapplystateByEntities(List<EAMApplyState> entities) {
        List ids =new ArrayList();
        for(EAMApplyState entity : entities){
            Serializable id=entity.getEamapplystateid();
            if(!ObjectUtils.isEmpty(id)){
                ids.add(id);
            }
        }
        if(ids.size()>0)
           return this.listByIds(ids);
        else
           return entities;
    }

}



