package cn.ibizlab.assetmanagement.core.apply.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import cn.ibizlab.assetmanagement.core.apply.domain.EAMApplyType;
import cn.ibizlab.assetmanagement.core.apply.filter.EAMApplyTypeSearchContext;
import cn.ibizlab.assetmanagement.core.apply.service.IEAMApplyTypeService;

import cn.ibizlab.assetmanagement.util.helper.CachedBeanCopier;
import cn.ibizlab.assetmanagement.util.helper.DEFieldCacheMap;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cn.ibizlab.assetmanagement.core.apply.mapper.EAMApplyTypeMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;
import org.springframework.util.StringUtils;

/**
 * 实体[维护申请类型] 服务对象接口实现
 */
@Slf4j
@Service("EAMApplyTypeServiceImpl")
public class EAMApplyTypeServiceImpl extends ServiceImpl<EAMApplyTypeMapper, EAMApplyType> implements IEAMApplyTypeService {

    @Autowired
    @Lazy
    protected cn.ibizlab.assetmanagement.core.apply.service.IEAMApplyService eamapplyService;
    @Autowired
    @Lazy
    protected cn.ibizlab.assetmanagement.core.apply.service.IEAMApplyStateModelService eamapplystatemodelService;

    protected int batchSize = 500;

    @Override
    @Transactional
    public boolean create(EAMApplyType et) {
        if(!this.retBool(this.baseMapper.insert(et)))
            return false;
        CachedBeanCopier.copy(get(et.getEamapplytypeid()),et);
        return true;
    }

    @Override
    public void createBatch(List<EAMApplyType> list) {
        this.saveBatch(list,batchSize);
    }

    @Override
    @Transactional
    public boolean update(EAMApplyType et) {
        if(!update(et,(Wrapper) et.getUpdateWrapper(true).eq("eamapplytypeid",et.getEamapplytypeid())))
            return false;
        CachedBeanCopier.copy(get(et.getEamapplytypeid()),et);
        return true;
    }

    @Override
    public void updateBatch(List<EAMApplyType> list) {
        updateBatchById(list,batchSize);
    }

    @Override
    @Transactional
    public boolean remove(String key) {
        boolean result=removeById(key);
        return result ;
    }

    @Override
    public void removeBatch(Collection<String> idList) {
        removeByIds(idList);
    }

    @Override
    @Transactional
    public EAMApplyType get(String key) {
        EAMApplyType et = getById(key);
        if(et==null){
            et=new EAMApplyType();
            et.setEamapplytypeid(key);
        }
        else{
        }
        return et;
    }

    @Override
    public EAMApplyType getDraft(EAMApplyType et) {
        return et;
    }

    @Override
    public boolean checkKey(EAMApplyType et) {
        return (!ObjectUtils.isEmpty(et.getEamapplytypeid()))&&(!Objects.isNull(this.getById(et.getEamapplytypeid())));
    }
    @Override
    @Transactional
    public boolean save(EAMApplyType et) {
        if(!saveOrUpdate(et))
            return false;
        return true;
    }

    @Override
    @Transactional
    public boolean saveOrUpdate(EAMApplyType et) {
        if (null == et) {
            return false;
        } else {
            return checkKey(et) ? this.update(et) : this.create(et);
        }
    }

    @Override
    public boolean saveBatch(Collection<EAMApplyType> list) {
        saveOrUpdateBatch(list,batchSize);
        return true;
    }

    @Override
    public void saveBatch(List<EAMApplyType> list) {
        saveOrUpdateBatch(list,batchSize);
    }


	@Override
    public List<EAMApplyType> selectByEamapplystatemodelid(String eamapplystatemodelid) {
        return baseMapper.selectByEamapplystatemodelid(eamapplystatemodelid);
    }

    @Override
    public void removeByEamapplystatemodelid(String eamapplystatemodelid) {
        this.remove(new QueryWrapper<EAMApplyType>().eq("eamapplystatemodelid",eamapplystatemodelid));
    }


    /**
     * 查询集合 DEFAULT
     */
    @Override
    public Page<EAMApplyType> searchDefault(EAMApplyTypeSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<EAMApplyType> pages=baseMapper.searchDefault(context.getPages(),context,context.getSelectCond());
        return new PageImpl<EAMApplyType>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }







    @Override
    public List<JSONObject> select(String sql, Map param){
        return this.baseMapper.selectBySQL(sql,param);
    }

    @Override
    @Transactional
    public boolean execute(String sql , Map param){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql,param);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }

    @Override
    public List<EAMApplyType> getEamapplytypeByIds(List<String> ids) {
         return this.listByIds(ids);
    }

    @Override
    public List<EAMApplyType> getEamapplytypeByEntities(List<EAMApplyType> entities) {
        List ids =new ArrayList();
        for(EAMApplyType entity : entities){
            Serializable id=entity.getEamapplytypeid();
            if(!ObjectUtils.isEmpty(id)){
                ids.add(id);
            }
        }
        if(ids.size()>0)
           return this.listByIds(ids);
        else
           return entities;
    }

}



