package cn.ibizlab.assetmanagement.core.asset.domain;

import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.math.BigInteger;
import java.util.HashMap;
import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import com.alibaba.fastjson.annotation.JSONField;
import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.annotation.JsonFormat;
import org.springframework.util.ObjectUtils;
import org.springframework.util.DigestUtils;
import cn.ibizlab.assetmanagement.util.domain.EntityBase;
import cn.ibizlab.assetmanagement.util.annotation.DEField;
import cn.ibizlab.assetmanagement.util.enums.DEPredefinedFieldType;
import cn.ibizlab.assetmanagement.util.enums.DEFieldDefaultValueType;
import java.io.Serializable;
import lombok.*;
import org.springframework.data.annotation.Transient;
import cn.ibizlab.assetmanagement.util.annotation.Audit;


import com.fasterxml.jackson.annotation.JsonIgnoreProperties;
import com.baomidou.mybatisplus.annotation.*;
import cn.ibizlab.assetmanagement.util.domain.EntityMP;

/**
 * 实体[资产]
 */
@Getter
@Setter
@NoArgsConstructor
@JsonIgnoreProperties(value = "handler")
@TableName(value = "ASSET",resultMap = "EAMAssetResultMap")
public class EAMAsset extends EntityMP implements Serializable {

    private static final long serialVersionUID = 1L;

    /**
     * 资产标识
     */
    @DEField(isKeyField=true)
    @TableId(value= "assetid",type=IdType.ASSIGN_UUID)
    @JSONField(name = "assetid")
    @JsonProperty("assetid")
    private String assetid;
    /**
     * 资产名称
     */
    @TableField(value = "assetname")
    @JSONField(name = "assetname")
    @JsonProperty("assetname")
    private String assetname;
    /**
     * 建立时间
     */
    @DEField(preType = DEPredefinedFieldType.CREATEDATE)
    @TableField(value = "createdate" , fill = FieldFill.INSERT)
    @JsonFormat(pattern="yyyy-MM-dd HH:mm:ss", locale = "zh" , timezone="GMT+8")
    @JSONField(name = "createdate" , format="yyyy-MM-dd HH:mm:ss")
    @JsonProperty("createdate")
    private Timestamp createdate;
    /**
     * 建立人
     */
    @DEField(preType = DEPredefinedFieldType.CREATEMAN)
    @TableField(value = "createman" , fill = FieldFill.INSERT)
    @JSONField(name = "createman")
    @JsonProperty("createman")
    private String createman;
    /**
     * 更新时间
     */
    @DEField(preType = DEPredefinedFieldType.UPDATEDATE)
    @TableField(value = "updatedate")
    @JsonFormat(pattern="yyyy-MM-dd HH:mm:ss", locale = "zh" , timezone="GMT+8")
    @JSONField(name = "updatedate" , format="yyyy-MM-dd HH:mm:ss")
    @JsonProperty("updatedate")
    private Timestamp updatedate;
    /**
     * 更新人
     */
    @DEField(preType = DEPredefinedFieldType.UPDATEMAN)
    @TableField(value = "updateman")
    @JSONField(name = "updateman")
    @JsonProperty("updateman")
    private String updateman;
    /**
     * 资产类型标识
     */
    @TableField(value = "eamassettypeid")
    @JSONField(name = "eamassettypeid")
    @JsonProperty("eamassettypeid")
    private String eamassettypeid;
    /**
     * 功能位置标识
     */
    @TableField(value = "eamlocationid")
    @JSONField(name = "eamlocationid")
    @JsonProperty("eamlocationid")
    private String eamlocationid;
    /**
     * 上级资产标识
     */
    @TableField(value = "passetid")
    @JSONField(name = "passetid")
    @JsonProperty("passetid")
    private String passetid;
    /**
     * 资产类型
     */
    @TableField(exist = false)
    @JSONField(name = "eamassettypename")
    @JsonProperty("eamassettypename")
    private String eamassettypename;
    /**
     * 功能位置
     */
    @TableField(exist = false)
    @JSONField(name = "eamlocationname")
    @JsonProperty("eamlocationname")
    private String eamlocationname;
    /**
     * 资产状态标识
     */
    @TableField(value = "eamassetstateid")
    @JSONField(name = "eamassetstateid")
    @JsonProperty("eamassetstateid")
    private String eamassetstateid;
    /**
     * 资产状态
     */
    @TableField(exist = false)
    @JSONField(name = "eamassetstatename")
    @JsonProperty("eamassetstatename")
    private String eamassetstatename;
    /**
     * 上级资产
     */
    @TableField(exist = false)
    @JSONField(name = "passetname")
    @JsonProperty("passetname")
    private String passetname;
    /**
     * 资产编号
     */
    @TableField(value = "assetnumber")
    @JSONField(name = "assetnumber")
    @JsonProperty("assetnumber")
    private String assetnumber;

    /**
     * 
     */
    @JsonIgnore
    @JSONField(serialize = false)
    @TableField(exist = false)
    private cn.ibizlab.assetmanagement.core.asset.domain.EAMAssetState eamassetstate;

    /**
     * 
     */
    @JsonIgnore
    @JSONField(serialize = false)
    @TableField(exist = false)
    private cn.ibizlab.assetmanagement.core.asset.domain.EAMAssetType eamassettype;

    /**
     * 
     */
    @JsonIgnore
    @JSONField(serialize = false)
    @TableField(exist = false)
    private cn.ibizlab.assetmanagement.core.asset.domain.EAMAsset passet;

    /**
     * 
     */
    @JsonIgnore
    @JSONField(serialize = false)
    @TableField(exist = false)
    private cn.ibizlab.assetmanagement.core.location.domain.EAMLocation eamlocation;



    /**
     * 设置 [资产名称]
     */
    public void setAssetname(String assetname){
        this.assetname = assetname ;
        this.modify("assetname",assetname);
    }

    /**
     * 设置 [资产类型标识]
     */
    public void setEamassettypeid(String eamassettypeid){
        this.eamassettypeid = eamassettypeid ;
        this.modify("eamassettypeid",eamassettypeid);
    }

    /**
     * 设置 [功能位置标识]
     */
    public void setEamlocationid(String eamlocationid){
        this.eamlocationid = eamlocationid ;
        this.modify("eamlocationid",eamlocationid);
    }

    /**
     * 设置 [上级资产标识]
     */
    public void setPassetid(String passetid){
        this.passetid = passetid ;
        this.modify("passetid",passetid);
    }

    /**
     * 设置 [资产状态标识]
     */
    public void setEamassetstateid(String eamassetstateid){
        this.eamassetstateid = eamassetstateid ;
        this.modify("eamassetstateid",eamassetstateid);
    }

    /**
     * 设置 [资产编号]
     */
    public void setAssetnumber(String assetnumber){
        this.assetnumber = assetnumber ;
        this.modify("assetnumber",assetnumber);
    }


}


