package cn.ibizlab.assetmanagement.core.asset.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import cn.ibizlab.assetmanagement.core.asset.domain.EAMAssetPartsGroup;
import cn.ibizlab.assetmanagement.core.asset.filter.EAMAssetPartsGroupSearchContext;
import cn.ibizlab.assetmanagement.core.asset.service.IEAMAssetPartsGroupService;

import cn.ibizlab.assetmanagement.util.helper.CachedBeanCopier;
import cn.ibizlab.assetmanagement.util.helper.DEFieldCacheMap;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cn.ibizlab.assetmanagement.core.asset.mapper.EAMAssetPartsGroupMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;
import org.springframework.util.StringUtils;

/**
 * 实体[资产备件包] 服务对象接口实现
 */
@Slf4j
@Service("EAMAssetPartsGroupServiceImpl")
public class EAMAssetPartsGroupServiceImpl extends ServiceImpl<EAMAssetPartsGroupMapper, EAMAssetPartsGroup> implements IEAMAssetPartsGroupService {

    @Autowired
    @Lazy
    protected cn.ibizlab.assetmanagement.core.asset.service.IEAMAssetTypeService eamassettypeService;
    @Autowired
    @Lazy
    protected cn.ibizlab.assetmanagement.core.asset.service.IEAMSparePartsGroupService eamsparepartsgroupService;

    protected int batchSize = 500;

    @Override
    @Transactional
    public boolean create(EAMAssetPartsGroup et) {
        fillParentData(et);
        if(!this.retBool(this.baseMapper.insert(et)))
            return false;
        CachedBeanCopier.copy(get(et.getEamassetpartsgroupid()),et);
        return true;
    }

    @Override
    public void createBatch(List<EAMAssetPartsGroup> list) {
        list.forEach(item->fillParentData(item));
        this.saveBatch(list,batchSize);
    }

    @Override
    @Transactional
    public boolean update(EAMAssetPartsGroup et) {
        fillParentData(et);
        if(!update(et,(Wrapper) et.getUpdateWrapper(true).eq("eamassetpartsgroupid",et.getEamassetpartsgroupid())))
            return false;
        CachedBeanCopier.copy(get(et.getEamassetpartsgroupid()),et);
        return true;
    }

    @Override
    public void updateBatch(List<EAMAssetPartsGroup> list) {
        list.forEach(item->fillParentData(item));
        updateBatchById(list,batchSize);
    }

    @Override
    @Transactional
    public boolean remove(String key) {
        boolean result=removeById(key);
        return result ;
    }

    @Override
    public void removeBatch(Collection<String> idList) {
        removeByIds(idList);
    }

    @Override
    @Transactional
    public EAMAssetPartsGroup get(String key) {
        EAMAssetPartsGroup et = getById(key);
        if(et==null){
            et=new EAMAssetPartsGroup();
            et.setEamassetpartsgroupid(key);
        }
        else{
        }
        return et;
    }

    @Override
    public EAMAssetPartsGroup getDraft(EAMAssetPartsGroup et) {
        fillParentData(et);
        return et;
    }

    @Override
    public boolean checkKey(EAMAssetPartsGroup et) {
        return (!ObjectUtils.isEmpty(et.getEamassetpartsgroupid()))&&(!Objects.isNull(this.getById(et.getEamassetpartsgroupid())));
    }
    @Override
    @Transactional
    public boolean save(EAMAssetPartsGroup et) {
        if(!saveOrUpdate(et))
            return false;
        return true;
    }

    @Override
    @Transactional
    public boolean saveOrUpdate(EAMAssetPartsGroup et) {
        if (null == et) {
            return false;
        } else {
            return checkKey(et) ? this.update(et) : this.create(et);
        }
    }

    @Override
    public boolean saveBatch(Collection<EAMAssetPartsGroup> list) {
        list.forEach(item->fillParentData(item));
        saveOrUpdateBatch(list,batchSize);
        return true;
    }

    @Override
    public void saveBatch(List<EAMAssetPartsGroup> list) {
        list.forEach(item->fillParentData(item));
        saveOrUpdateBatch(list,batchSize);
    }


	@Override
    public List<EAMAssetPartsGroup> selectByEamassettypeid(String eamassettypeid) {
        return baseMapper.selectByEamassettypeid(eamassettypeid);
    }

    @Override
    public void removeByEamassettypeid(String eamassettypeid) {
        this.remove(new QueryWrapper<EAMAssetPartsGroup>().eq("eamassettypeid",eamassettypeid));
    }

	@Override
    public List<EAMAssetPartsGroup> selectByEamsparepartsgroupid(String eamsparepartsgroupid) {
        return baseMapper.selectByEamsparepartsgroupid(eamsparepartsgroupid);
    }

    @Override
    public void removeByEamsparepartsgroupid(String eamsparepartsgroupid) {
        this.remove(new QueryWrapper<EAMAssetPartsGroup>().eq("eamsparepartsgroupid",eamsparepartsgroupid));
    }


    /**
     * 查询集合 DEFAULT
     */
    @Override
    public Page<EAMAssetPartsGroup> searchDefault(EAMAssetPartsGroupSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<EAMAssetPartsGroup> pages=baseMapper.searchDefault(context.getPages(),context,context.getSelectCond());
        return new PageImpl<EAMAssetPartsGroup>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }



    /**
     * 为当前实体填充父数据（外键值文本、外键值附加数据）
     * @param et
     */
    private void fillParentData(EAMAssetPartsGroup et){
        //实体关系[DER1N_EAMASSETPARTSGROUP_EAMASSETTYPE_EAMASSETTYPEID]
        if(!ObjectUtils.isEmpty(et.getEamassettypeid())){
            cn.ibizlab.assetmanagement.core.asset.domain.EAMAssetType eamassettype=et.getEamassettype();
            if(ObjectUtils.isEmpty(eamassettype)){
                cn.ibizlab.assetmanagement.core.asset.domain.EAMAssetType majorEntity=eamassettypeService.get(et.getEamassettypeid());
                et.setEamassettype(majorEntity);
                eamassettype=majorEntity;
            }
            et.setEamassettypename(eamassettype.getEamassettypename());
        }
        //实体关系[DER1N_EAMASSETPARTSGROUP_EAMSPAREPARTSGROUP_EAMSPAREPARTSGROUPID]
        if(!ObjectUtils.isEmpty(et.getEamsparepartsgroupid())){
            cn.ibizlab.assetmanagement.core.asset.domain.EAMSparePartsGroup eamsparepartsgroup=et.getEamsparepartsgroup();
            if(ObjectUtils.isEmpty(eamsparepartsgroup)){
                cn.ibizlab.assetmanagement.core.asset.domain.EAMSparePartsGroup majorEntity=eamsparepartsgroupService.get(et.getEamsparepartsgroupid());
                et.setEamsparepartsgroup(majorEntity);
                eamsparepartsgroup=majorEntity;
            }
            et.setEamsparepartsgroupname(eamsparepartsgroup.getEamsparepartsgroupname());
        }
    }




    @Override
    public List<JSONObject> select(String sql, Map param){
        return this.baseMapper.selectBySQL(sql,param);
    }

    @Override
    @Transactional
    public boolean execute(String sql , Map param){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql,param);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }

    @Override
    public List<EAMAssetPartsGroup> getEamassetpartsgroupByIds(List<String> ids) {
         return this.listByIds(ids);
    }

    @Override
    public List<EAMAssetPartsGroup> getEamassetpartsgroupByEntities(List<EAMAssetPartsGroup> entities) {
        List ids =new ArrayList();
        for(EAMAssetPartsGroup entity : entities){
            Serializable id=entity.getEamassetpartsgroupid();
            if(!ObjectUtils.isEmpty(id)){
                ids.add(id);
            }
        }
        if(ids.size()>0)
           return this.listByIds(ids);
        else
           return entities;
    }

}



