package cn.ibizlab.assetmanagement.core.asset.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import cn.ibizlab.assetmanagement.core.asset.domain.EAMAssetStateModelLine;
import cn.ibizlab.assetmanagement.core.asset.filter.EAMAssetStateModelLineSearchContext;
import cn.ibizlab.assetmanagement.core.asset.service.IEAMAssetStateModelLineService;

import cn.ibizlab.assetmanagement.util.helper.CachedBeanCopier;
import cn.ibizlab.assetmanagement.util.helper.DEFieldCacheMap;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cn.ibizlab.assetmanagement.core.asset.mapper.EAMAssetStateModelLineMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;
import org.springframework.util.StringUtils;

/**
 * 实体[资产状态模型明细] 服务对象接口实现
 */
@Slf4j
@Service("EAMAssetStateModelLineServiceImpl")
public class EAMAssetStateModelLineServiceImpl extends ServiceImpl<EAMAssetStateModelLineMapper, EAMAssetStateModelLine> implements IEAMAssetStateModelLineService {

    @Autowired
    @Lazy
    protected cn.ibizlab.assetmanagement.core.asset.service.IEAMAssetStateModelService eamassetstatemodelService;
    @Autowired
    @Lazy
    protected cn.ibizlab.assetmanagement.core.asset.service.IEAMAssetStateService eamassetstateService;

    protected int batchSize = 500;

    @Override
    @Transactional
    public boolean create(EAMAssetStateModelLine et) {
        fillParentData(et);
        if(!this.retBool(this.baseMapper.insert(et)))
            return false;
        CachedBeanCopier.copy(get(et.getEamassetstatemodellineid()),et);
        return true;
    }

    @Override
    public void createBatch(List<EAMAssetStateModelLine> list) {
        list.forEach(item->fillParentData(item));
        this.saveBatch(list,batchSize);
    }

    @Override
    @Transactional
    public boolean update(EAMAssetStateModelLine et) {
        fillParentData(et);
        if(!update(et,(Wrapper) et.getUpdateWrapper(true).eq("eamassetstatemodellineid",et.getEamassetstatemodellineid())))
            return false;
        CachedBeanCopier.copy(get(et.getEamassetstatemodellineid()),et);
        return true;
    }

    @Override
    public void updateBatch(List<EAMAssetStateModelLine> list) {
        list.forEach(item->fillParentData(item));
        updateBatchById(list,batchSize);
    }

    @Override
    @Transactional
    public boolean remove(String key) {
        boolean result=removeById(key);
        return result ;
    }

    @Override
    public void removeBatch(Collection<String> idList) {
        removeByIds(idList);
    }

    @Override
    @Transactional
    public EAMAssetStateModelLine get(String key) {
        EAMAssetStateModelLine et = getById(key);
        if(et==null){
            et=new EAMAssetStateModelLine();
            et.setEamassetstatemodellineid(key);
        }
        else{
        }
        return et;
    }

    @Override
    public EAMAssetStateModelLine getDraft(EAMAssetStateModelLine et) {
        fillParentData(et);
        return et;
    }

    @Override
    public boolean checkKey(EAMAssetStateModelLine et) {
        return (!ObjectUtils.isEmpty(et.getEamassetstatemodellineid()))&&(!Objects.isNull(this.getById(et.getEamassetstatemodellineid())));
    }
    @Override
    @Transactional
    public boolean save(EAMAssetStateModelLine et) {
        if(!saveOrUpdate(et))
            return false;
        return true;
    }

    @Override
    @Transactional
    public boolean saveOrUpdate(EAMAssetStateModelLine et) {
        if (null == et) {
            return false;
        } else {
            return checkKey(et) ? this.update(et) : this.create(et);
        }
    }

    @Override
    public boolean saveBatch(Collection<EAMAssetStateModelLine> list) {
        list.forEach(item->fillParentData(item));
        saveOrUpdateBatch(list,batchSize);
        return true;
    }

    @Override
    public void saveBatch(List<EAMAssetStateModelLine> list) {
        list.forEach(item->fillParentData(item));
        saveOrUpdateBatch(list,batchSize);
    }


	@Override
    public List<EAMAssetStateModelLine> selectByEamassetstatemodelid(String eamassetstatemodelid) {
        return baseMapper.selectByEamassetstatemodelid(eamassetstatemodelid);
    }

    @Override
    public void removeByEamassetstatemodelid(String eamassetstatemodelid) {
        this.remove(new QueryWrapper<EAMAssetStateModelLine>().eq("eamassetstatemodelid",eamassetstatemodelid));
    }

	@Override
    public List<EAMAssetStateModelLine> selectByEamassetstateid(String eamassetstateid) {
        return baseMapper.selectByEamassetstateid(eamassetstateid);
    }

    @Override
    public void removeByEamassetstateid(String eamassetstateid) {
        this.remove(new QueryWrapper<EAMAssetStateModelLine>().eq("eamassetstateid",eamassetstateid));
    }


    /**
     * 查询集合 DEFAULT
     */
    @Override
    public Page<EAMAssetStateModelLine> searchDefault(EAMAssetStateModelLineSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<EAMAssetStateModelLine> pages=baseMapper.searchDefault(context.getPages(),context,context.getSelectCond());
        return new PageImpl<EAMAssetStateModelLine>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }



    /**
     * 为当前实体填充父数据（外键值文本、外键值附加数据）
     * @param et
     */
    private void fillParentData(EAMAssetStateModelLine et){
        //实体关系[DER1N_EAMASSETSTATEMODELLINE_EAMASSETSTATEMODEL_EAMASSETSTATEMODELID]
        if(!ObjectUtils.isEmpty(et.getEamassetstatemodelid())){
            cn.ibizlab.assetmanagement.core.asset.domain.EAMAssetStateModel eamassetstatemodel=et.getEamassetstatemodel();
            if(ObjectUtils.isEmpty(eamassetstatemodel)){
                cn.ibizlab.assetmanagement.core.asset.domain.EAMAssetStateModel majorEntity=eamassetstatemodelService.get(et.getEamassetstatemodelid());
                et.setEamassetstatemodel(majorEntity);
                eamassetstatemodel=majorEntity;
            }
            et.setEamassetstatemodelname(eamassetstatemodel.getEamassetstatemodelname());
        }
        //实体关系[DER1N_EAMASSETSTATEMODELLINE_EAMASSETSTATE_EAMASSETSTATEID]
        if(!ObjectUtils.isEmpty(et.getEamassetstateid())){
            cn.ibizlab.assetmanagement.core.asset.domain.EAMAssetState eamassetstate=et.getEamassetstate();
            if(ObjectUtils.isEmpty(eamassetstate)){
                cn.ibizlab.assetmanagement.core.asset.domain.EAMAssetState majorEntity=eamassetstateService.get(et.getEamassetstateid());
                et.setEamassetstate(majorEntity);
                eamassetstate=majorEntity;
            }
            et.setEamassetstatename(eamassetstate.getEamassetstatename());
        }
    }




    @Override
    public List<JSONObject> select(String sql, Map param){
        return this.baseMapper.selectBySQL(sql,param);
    }

    @Override
    @Transactional
    public boolean execute(String sql , Map param){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql,param);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }

    @Override
    public List<EAMAssetStateModelLine> getEamassetstatemodellineByIds(List<String> ids) {
         return this.listByIds(ids);
    }

    @Override
    public List<EAMAssetStateModelLine> getEamassetstatemodellineByEntities(List<EAMAssetStateModelLine> entities) {
        List ids =new ArrayList();
        for(EAMAssetStateModelLine entity : entities){
            Serializable id=entity.getEamassetstatemodellineid();
            if(!ObjectUtils.isEmpty(id)){
                ids.add(id);
            }
        }
        if(ids.size()>0)
           return this.listByIds(ids);
        else
           return entities;
    }

}



