package cn.ibizlab.assetmanagement.core.asset.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import cn.ibizlab.assetmanagement.core.asset.domain.EAMAssetStateModel;
import cn.ibizlab.assetmanagement.core.asset.filter.EAMAssetStateModelSearchContext;
import cn.ibizlab.assetmanagement.core.asset.service.IEAMAssetStateModelService;

import cn.ibizlab.assetmanagement.util.helper.CachedBeanCopier;
import cn.ibizlab.assetmanagement.util.helper.DEFieldCacheMap;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cn.ibizlab.assetmanagement.core.asset.mapper.EAMAssetStateModelMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;
import org.springframework.util.StringUtils;

/**
 * 实体[资产状态模型] 服务对象接口实现
 */
@Slf4j
@Service("EAMAssetStateModelServiceImpl")
public class EAMAssetStateModelServiceImpl extends ServiceImpl<EAMAssetStateModelMapper, EAMAssetStateModel> implements IEAMAssetStateModelService {

    @Autowired
    @Lazy
    protected cn.ibizlab.assetmanagement.core.asset.service.IEAMAssetStateModelLineService eamassetstatemodellineService;
    @Autowired
    @Lazy
    protected cn.ibizlab.assetmanagement.core.asset.service.IEAMAssetTypeService eamassettypeService;

    protected int batchSize = 500;

    @Override
    @Transactional
    public boolean create(EAMAssetStateModel et) {
        if(!this.retBool(this.baseMapper.insert(et)))
            return false;
        CachedBeanCopier.copy(get(et.getEamassetstatemodelid()),et);
        return true;
    }

    @Override
    public void createBatch(List<EAMAssetStateModel> list) {
        this.saveBatch(list,batchSize);
    }

    @Override
    @Transactional
    public boolean update(EAMAssetStateModel et) {
        if(!update(et,(Wrapper) et.getUpdateWrapper(true).eq("eamassetstatemodelid",et.getEamassetstatemodelid())))
            return false;
        CachedBeanCopier.copy(get(et.getEamassetstatemodelid()),et);
        return true;
    }

    @Override
    public void updateBatch(List<EAMAssetStateModel> list) {
        updateBatchById(list,batchSize);
    }

    @Override
    @Transactional
    public boolean remove(String key) {
        boolean result=removeById(key);
        return result ;
    }

    @Override
    public void removeBatch(Collection<String> idList) {
        removeByIds(idList);
    }

    @Override
    @Transactional
    public EAMAssetStateModel get(String key) {
        EAMAssetStateModel et = getById(key);
        if(et==null){
            et=new EAMAssetStateModel();
            et.setEamassetstatemodelid(key);
        }
        else{
        }
        return et;
    }

    @Override
    public EAMAssetStateModel getDraft(EAMAssetStateModel et) {
        return et;
    }

    @Override
    public boolean checkKey(EAMAssetStateModel et) {
        return (!ObjectUtils.isEmpty(et.getEamassetstatemodelid()))&&(!Objects.isNull(this.getById(et.getEamassetstatemodelid())));
    }
    @Override
    @Transactional
    public boolean save(EAMAssetStateModel et) {
        if(!saveOrUpdate(et))
            return false;
        return true;
    }

    @Override
    @Transactional
    public boolean saveOrUpdate(EAMAssetStateModel et) {
        if (null == et) {
            return false;
        } else {
            return checkKey(et) ? this.update(et) : this.create(et);
        }
    }

    @Override
    public boolean saveBatch(Collection<EAMAssetStateModel> list) {
        saveOrUpdateBatch(list,batchSize);
        return true;
    }

    @Override
    public void saveBatch(List<EAMAssetStateModel> list) {
        saveOrUpdateBatch(list,batchSize);
    }



    /**
     * 查询集合 DEFAULT
     */
    @Override
    public Page<EAMAssetStateModel> searchDefault(EAMAssetStateModelSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<EAMAssetStateModel> pages=baseMapper.searchDefault(context.getPages(),context,context.getSelectCond());
        return new PageImpl<EAMAssetStateModel>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }







    @Override
    public List<JSONObject> select(String sql, Map param){
        return this.baseMapper.selectBySQL(sql,param);
    }

    @Override
    @Transactional
    public boolean execute(String sql , Map param){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql,param);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }

    @Override
    public List<EAMAssetStateModel> getEamassetstatemodelByIds(List<String> ids) {
         return this.listByIds(ids);
    }

    @Override
    public List<EAMAssetStateModel> getEamassetstatemodelByEntities(List<EAMAssetStateModel> entities) {
        List ids =new ArrayList();
        for(EAMAssetStateModel entity : entities){
            Serializable id=entity.getEamassetstatemodelid();
            if(!ObjectUtils.isEmpty(id)){
                ids.add(id);
            }
        }
        if(ids.size()>0)
           return this.listByIds(ids);
        else
           return entities;
    }

}



