package cn.ibizlab.assetmanagement.core.asset.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import cn.ibizlab.assetmanagement.core.asset.domain.EAMSparePartsGroup;
import cn.ibizlab.assetmanagement.core.asset.filter.EAMSparePartsGroupSearchContext;
import cn.ibizlab.assetmanagement.core.asset.service.IEAMSparePartsGroupService;

import cn.ibizlab.assetmanagement.util.helper.CachedBeanCopier;
import cn.ibizlab.assetmanagement.util.helper.DEFieldCacheMap;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cn.ibizlab.assetmanagement.core.asset.mapper.EAMSparePartsGroupMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;
import org.springframework.util.StringUtils;

/**
 * 实体[备件包] 服务对象接口实现
 */
@Slf4j
@Service("EAMSparePartsGroupServiceImpl")
public class EAMSparePartsGroupServiceImpl extends ServiceImpl<EAMSparePartsGroupMapper, EAMSparePartsGroup> implements IEAMSparePartsGroupService {

    @Autowired
    @Lazy
    protected cn.ibizlab.assetmanagement.core.asset.service.IEAMAssetPartsGroupService eamassetpartsgroupService;
    @Autowired
    @Lazy
    protected cn.ibizlab.assetmanagement.core.asset.service.IEAMSparePartsPROService eamsparepartsproService;

    protected int batchSize = 500;

    @Override
    @Transactional
    public boolean create(EAMSparePartsGroup et) {
        if(!this.retBool(this.baseMapper.insert(et)))
            return false;
        CachedBeanCopier.copy(get(et.getEamsparepartsgroupid()),et);
        return true;
    }

    @Override
    public void createBatch(List<EAMSparePartsGroup> list) {
        this.saveBatch(list,batchSize);
    }

    @Override
    @Transactional
    public boolean update(EAMSparePartsGroup et) {
        if(!update(et,(Wrapper) et.getUpdateWrapper(true).eq("eamsparepartsgroupid",et.getEamsparepartsgroupid())))
            return false;
        CachedBeanCopier.copy(get(et.getEamsparepartsgroupid()),et);
        return true;
    }

    @Override
    public void updateBatch(List<EAMSparePartsGroup> list) {
        updateBatchById(list,batchSize);
    }

    @Override
    @Transactional
    public boolean remove(String key) {
        boolean result=removeById(key);
        return result ;
    }

    @Override
    public void removeBatch(Collection<String> idList) {
        removeByIds(idList);
    }

    @Override
    @Transactional
    public EAMSparePartsGroup get(String key) {
        EAMSparePartsGroup et = getById(key);
        if(et==null){
            et=new EAMSparePartsGroup();
            et.setEamsparepartsgroupid(key);
        }
        else{
        }
        return et;
    }

    @Override
    public EAMSparePartsGroup getDraft(EAMSparePartsGroup et) {
        return et;
    }

    @Override
    public boolean checkKey(EAMSparePartsGroup et) {
        return (!ObjectUtils.isEmpty(et.getEamsparepartsgroupid()))&&(!Objects.isNull(this.getById(et.getEamsparepartsgroupid())));
    }
    @Override
    @Transactional
    public boolean save(EAMSparePartsGroup et) {
        if(!saveOrUpdate(et))
            return false;
        return true;
    }

    @Override
    @Transactional
    public boolean saveOrUpdate(EAMSparePartsGroup et) {
        if (null == et) {
            return false;
        } else {
            return checkKey(et) ? this.update(et) : this.create(et);
        }
    }

    @Override
    public boolean saveBatch(Collection<EAMSparePartsGroup> list) {
        saveOrUpdateBatch(list,batchSize);
        return true;
    }

    @Override
    public void saveBatch(List<EAMSparePartsGroup> list) {
        saveOrUpdateBatch(list,batchSize);
    }



    /**
     * 查询集合 DEFAULT
     */
    @Override
    public Page<EAMSparePartsGroup> searchDefault(EAMSparePartsGroupSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<EAMSparePartsGroup> pages=baseMapper.searchDefault(context.getPages(),context,context.getSelectCond());
        return new PageImpl<EAMSparePartsGroup>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }







    @Override
    public List<JSONObject> select(String sql, Map param){
        return this.baseMapper.selectBySQL(sql,param);
    }

    @Override
    @Transactional
    public boolean execute(String sql , Map param){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql,param);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }

    @Override
    public List<EAMSparePartsGroup> getEamsparepartsgroupByIds(List<String> ids) {
         return this.listByIds(ids);
    }

    @Override
    public List<EAMSparePartsGroup> getEamsparepartsgroupByEntities(List<EAMSparePartsGroup> entities) {
        List ids =new ArrayList();
        for(EAMSparePartsGroup entity : entities){
            Serializable id=entity.getEamsparepartsgroupid();
            if(!ObjectUtils.isEmpty(id)){
                ids.add(id);
            }
        }
        if(ids.size()>0)
           return this.listByIds(ids);
        else
           return entities;
    }

}



