package cn.ibizlab.assetmanagement.core.asset.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import cn.ibizlab.assetmanagement.core.asset.domain.EAMSparePartsPRO;
import cn.ibizlab.assetmanagement.core.asset.filter.EAMSparePartsPROSearchContext;
import cn.ibizlab.assetmanagement.core.asset.service.IEAMSparePartsPROService;

import cn.ibizlab.assetmanagement.util.helper.CachedBeanCopier;
import cn.ibizlab.assetmanagement.util.helper.DEFieldCacheMap;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cn.ibizlab.assetmanagement.core.asset.mapper.EAMSparePartsPROMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;
import org.springframework.util.StringUtils;

/**
 * 实体[备件包物料] 服务对象接口实现
 */
@Slf4j
@Service("EAMSparePartsPROServiceImpl")
public class EAMSparePartsPROServiceImpl extends ServiceImpl<EAMSparePartsPROMapper, EAMSparePartsPRO> implements IEAMSparePartsPROService {

    @Autowired
    @Lazy
    protected cn.ibizlab.assetmanagement.core.asset.service.IEAMPRODUCTService eamproductService;
    @Autowired
    @Lazy
    protected cn.ibizlab.assetmanagement.core.asset.service.IEAMSparePartsGroupService eamsparepartsgroupService;

    protected int batchSize = 500;

    @Override
    @Transactional
    public boolean create(EAMSparePartsPRO et) {
        fillParentData(et);
        if(!this.retBool(this.baseMapper.insert(et)))
            return false;
        CachedBeanCopier.copy(get(et.getEamsparepartsproid()),et);
        return true;
    }

    @Override
    public void createBatch(List<EAMSparePartsPRO> list) {
        list.forEach(item->fillParentData(item));
        this.saveBatch(list,batchSize);
    }

    @Override
    @Transactional
    public boolean update(EAMSparePartsPRO et) {
        fillParentData(et);
        if(!update(et,(Wrapper) et.getUpdateWrapper(true).eq("eamsparepartsproid",et.getEamsparepartsproid())))
            return false;
        CachedBeanCopier.copy(get(et.getEamsparepartsproid()),et);
        return true;
    }

    @Override
    public void updateBatch(List<EAMSparePartsPRO> list) {
        list.forEach(item->fillParentData(item));
        updateBatchById(list,batchSize);
    }

    @Override
    @Transactional
    public boolean remove(String key) {
        boolean result=removeById(key);
        return result ;
    }

    @Override
    public void removeBatch(Collection<String> idList) {
        removeByIds(idList);
    }

    @Override
    @Transactional
    public EAMSparePartsPRO get(String key) {
        EAMSparePartsPRO et = getById(key);
        if(et==null){
            et=new EAMSparePartsPRO();
            et.setEamsparepartsproid(key);
        }
        else{
        }
        return et;
    }

    @Override
    public EAMSparePartsPRO getDraft(EAMSparePartsPRO et) {
        fillParentData(et);
        return et;
    }

    @Override
    public boolean checkKey(EAMSparePartsPRO et) {
        return (!ObjectUtils.isEmpty(et.getEamsparepartsproid()))&&(!Objects.isNull(this.getById(et.getEamsparepartsproid())));
    }
    @Override
    @Transactional
    public boolean save(EAMSparePartsPRO et) {
        if(!saveOrUpdate(et))
            return false;
        return true;
    }

    @Override
    @Transactional
    public boolean saveOrUpdate(EAMSparePartsPRO et) {
        if (null == et) {
            return false;
        } else {
            return checkKey(et) ? this.update(et) : this.create(et);
        }
    }

    @Override
    public boolean saveBatch(Collection<EAMSparePartsPRO> list) {
        list.forEach(item->fillParentData(item));
        saveOrUpdateBatch(list,batchSize);
        return true;
    }

    @Override
    public void saveBatch(List<EAMSparePartsPRO> list) {
        list.forEach(item->fillParentData(item));
        saveOrUpdateBatch(list,batchSize);
    }


	@Override
    public List<EAMSparePartsPRO> selectByProductid(String productid) {
        return baseMapper.selectByProductid(productid);
    }

    @Override
    public void removeByProductid(String productid) {
        this.remove(new QueryWrapper<EAMSparePartsPRO>().eq("productid",productid));
    }

	@Override
    public List<EAMSparePartsPRO> selectByEamsparepartsgroupid(String eamsparepartsgroupid) {
        return baseMapper.selectByEamsparepartsgroupid(eamsparepartsgroupid);
    }

    @Override
    public void removeByEamsparepartsgroupid(String eamsparepartsgroupid) {
        this.remove(new QueryWrapper<EAMSparePartsPRO>().eq("eamsparepartsgroupid",eamsparepartsgroupid));
    }


    /**
     * 查询集合 DEFAULT
     */
    @Override
    public Page<EAMSparePartsPRO> searchDefault(EAMSparePartsPROSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<EAMSparePartsPRO> pages=baseMapper.searchDefault(context.getPages(),context,context.getSelectCond());
        return new PageImpl<EAMSparePartsPRO>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }



    /**
     * 为当前实体填充父数据（外键值文本、外键值附加数据）
     * @param et
     */
    private void fillParentData(EAMSparePartsPRO et){
        //实体关系[DER1N_EAMSPAREPARTSPRO_EAMPRODUCT_PRODUCTID]
        if(!ObjectUtils.isEmpty(et.getProductid())){
            cn.ibizlab.assetmanagement.core.asset.domain.EAMPRODUCT product=et.getProduct();
            if(ObjectUtils.isEmpty(product)){
                cn.ibizlab.assetmanagement.core.asset.domain.EAMPRODUCT majorEntity=eamproductService.get(et.getProductid());
                et.setProduct(majorEntity);
                product=majorEntity;
            }
            et.setProductname(product.getProductname());
            et.setProductnumber(product.getProductnumber());
        }
        //实体关系[DER1N_EAMSPAREPARTSPRO_EAMSPAREPARTSGROUP_EAMSPAREPARTSGROUPID]
        if(!ObjectUtils.isEmpty(et.getEamsparepartsgroupid())){
            cn.ibizlab.assetmanagement.core.asset.domain.EAMSparePartsGroup eamsparepartsgroup=et.getEamsparepartsgroup();
            if(ObjectUtils.isEmpty(eamsparepartsgroup)){
                cn.ibizlab.assetmanagement.core.asset.domain.EAMSparePartsGroup majorEntity=eamsparepartsgroupService.get(et.getEamsparepartsgroupid());
                et.setEamsparepartsgroup(majorEntity);
                eamsparepartsgroup=majorEntity;
            }
            et.setEamsparepartsgroupname(eamsparepartsgroup.getEamsparepartsgroupname());
        }
    }




    @Override
    public List<JSONObject> select(String sql, Map param){
        return this.baseMapper.selectBySQL(sql,param);
    }

    @Override
    @Transactional
    public boolean execute(String sql , Map param){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql,param);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }

    @Override
    public List<EAMSparePartsPRO> getEamsparepartsproByIds(List<String> ids) {
         return this.listByIds(ids);
    }

    @Override
    public List<EAMSparePartsPRO> getEamsparepartsproByEntities(List<EAMSparePartsPRO> entities) {
        List ids =new ArrayList();
        for(EAMSparePartsPRO entity : entities){
            Serializable id=entity.getEamsparepartsproid();
            if(!ObjectUtils.isEmpty(id)){
                ids.add(id);
            }
        }
        if(ids.size()>0)
           return this.listByIds(ids);
        else
           return entities;
    }

}



