package cn.ibizlab.assetmanagement.core.location.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import cn.ibizlab.assetmanagement.core.location.domain.EAMLocation;
import cn.ibizlab.assetmanagement.core.location.filter.EAMLocationSearchContext;
import cn.ibizlab.assetmanagement.core.location.service.IEAMLocationService;

import cn.ibizlab.assetmanagement.util.helper.CachedBeanCopier;
import cn.ibizlab.assetmanagement.util.helper.DEFieldCacheMap;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cn.ibizlab.assetmanagement.core.location.mapper.EAMLocationMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;
import org.springframework.util.StringUtils;

/**
 * 实体[功能位置] 服务对象接口实现
 */
@Slf4j
@Service("EAMLocationServiceImpl")
public class EAMLocationServiceImpl extends ServiceImpl<EAMLocationMapper, EAMLocation> implements IEAMLocationService {

    @Autowired
    @Lazy
    protected cn.ibizlab.assetmanagement.core.apply.service.IEAMApplyService eamapplyService;
    @Autowired
    @Lazy
    protected cn.ibizlab.assetmanagement.core.asset.service.IEAMAssetService eamassetService;

    protected cn.ibizlab.assetmanagement.core.location.service.IEAMLocationService eamlocationService = this;
    @Autowired
    @Lazy
    protected cn.ibizlab.assetmanagement.core.plan.service.IEAMPlanLineService eamplanlineService;
    @Autowired
    @Lazy
    protected cn.ibizlab.assetmanagement.core.plan.service.IEAMPlanService eamplanService;
    @Autowired
    @Lazy
    protected cn.ibizlab.assetmanagement.core.workorder.service.IEAMWorkOrderService eamworkorderService;
    @Autowired
    @Lazy
    protected cn.ibizlab.assetmanagement.core.location.service.IEAMLocationStateService eamlocationstateService;
    @Autowired
    @Lazy
    protected cn.ibizlab.assetmanagement.core.location.service.IEAMLocationTypeService eamlocationtypeService;

    protected int batchSize = 500;

    @Override
    @Transactional
    public boolean create(EAMLocation et) {
        fillParentData(et);
        if(!this.retBool(this.baseMapper.insert(et)))
            return false;
        CachedBeanCopier.copy(get(et.getEamlocationid()),et);
        return true;
    }

    @Override
    public void createBatch(List<EAMLocation> list) {
        list.forEach(item->fillParentData(item));
        this.saveBatch(list,batchSize);
    }

    @Override
    @Transactional
    public boolean update(EAMLocation et) {
        fillParentData(et);
        if(!update(et,(Wrapper) et.getUpdateWrapper(true).eq("eamlocationid",et.getEamlocationid())))
            return false;
        CachedBeanCopier.copy(get(et.getEamlocationid()),et);
        return true;
    }

    @Override
    public void updateBatch(List<EAMLocation> list) {
        list.forEach(item->fillParentData(item));
        updateBatchById(list,batchSize);
    }

    @Override
    @Transactional
    public boolean remove(String key) {
        boolean result=removeById(key);
        return result ;
    }

    @Override
    public void removeBatch(Collection<String> idList) {
        removeByIds(idList);
    }

    @Override
    @Transactional
    public EAMLocation get(String key) {
        EAMLocation et = getById(key);
        if(et==null){
            et=new EAMLocation();
            et.setEamlocationid(key);
        }
        else{
        }
        return et;
    }

    @Override
    public EAMLocation getDraft(EAMLocation et) {
        fillParentData(et);
        return et;
    }

    @Override
    public boolean checkKey(EAMLocation et) {
        return (!ObjectUtils.isEmpty(et.getEamlocationid()))&&(!Objects.isNull(this.getById(et.getEamlocationid())));
    }
    @Override
    @Transactional
    public boolean save(EAMLocation et) {
        if(!saveOrUpdate(et))
            return false;
        return true;
    }

    @Override
    @Transactional
    public boolean saveOrUpdate(EAMLocation et) {
        if (null == et) {
            return false;
        } else {
            return checkKey(et) ? this.update(et) : this.create(et);
        }
    }

    @Override
    public boolean saveBatch(Collection<EAMLocation> list) {
        list.forEach(item->fillParentData(item));
        saveOrUpdateBatch(list,batchSize);
        return true;
    }

    @Override
    public void saveBatch(List<EAMLocation> list) {
        list.forEach(item->fillParentData(item));
        saveOrUpdateBatch(list,batchSize);
    }


	@Override
    public List<EAMLocation> selectByEamlocationstateid(String eamlocationstateid) {
        return baseMapper.selectByEamlocationstateid(eamlocationstateid);
    }

    @Override
    public void removeByEamlocationstateid(String eamlocationstateid) {
        this.remove(new QueryWrapper<EAMLocation>().eq("eamlocationstateid",eamlocationstateid));
    }

	@Override
    public List<EAMLocation> selectByEamlocationtypeid(String eamlocationtypeid) {
        return baseMapper.selectByEamlocationtypeid(eamlocationtypeid);
    }

    @Override
    public void removeByEamlocationtypeid(String eamlocationtypeid) {
        this.remove(new QueryWrapper<EAMLocation>().eq("eamlocationtypeid",eamlocationtypeid));
    }

	@Override
    public List<EAMLocation> selectByPeamlocationid(String eamlocationid) {
        return baseMapper.selectByPeamlocationid(eamlocationid);
    }

    @Override
    public void removeByPeamlocationid(String eamlocationid) {
        this.remove(new QueryWrapper<EAMLocation>().eq("peamlocationid",eamlocationid));
    }


    /**
     * 查询集合 子位置（树）
     */
    @Override
    public Page<EAMLocation> searchChild(EAMLocationSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<EAMLocation> pages=baseMapper.searchChild(context.getPages(),context,context.getSelectCond());
        return new PageImpl<EAMLocation>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }

    /**
     * 查询集合 下级位置
     */
    @Override
    public Page<EAMLocation> searchChildLocation(EAMLocationSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<EAMLocation> pages=baseMapper.searchChildLocation(context.getPages(),context,context.getSelectCond());
        return new PageImpl<EAMLocation>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }

    /**
     * 查询集合 DEFAULT
     */
    @Override
    public Page<EAMLocation> searchDefault(EAMLocationSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<EAMLocation> pages=baseMapper.searchDefault(context.getPages(),context,context.getSelectCond());
        return new PageImpl<EAMLocation>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }

    /**
     * 查询集合 顶级位置
     */
    @Override
    public Page<EAMLocation> searchTop(EAMLocationSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<EAMLocation> pages=baseMapper.searchTop(context.getPages(),context,context.getSelectCond());
        return new PageImpl<EAMLocation>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }



    /**
     * 为当前实体填充父数据（外键值文本、外键值附加数据）
     * @param et
     */
    private void fillParentData(EAMLocation et){
        //实体关系[DER1N_EAMLOCATION_EAMLOCATIONSTATE_EAMLOCATIONSTATEID]
        if(!ObjectUtils.isEmpty(et.getEamlocationstateid())){
            cn.ibizlab.assetmanagement.core.location.domain.EAMLocationState eamlocationstate=et.getEamlocationstate();
            if(ObjectUtils.isEmpty(eamlocationstate)){
                cn.ibizlab.assetmanagement.core.location.domain.EAMLocationState majorEntity=eamlocationstateService.get(et.getEamlocationstateid());
                et.setEamlocationstate(majorEntity);
                eamlocationstate=majorEntity;
            }
            et.setEamlocationstatename(eamlocationstate.getEamlocationstatename());
        }
        //实体关系[DER1N_EAMLOCATION_EAMLOCATIONTYPE_EAMLOCATIONTYPEID]
        if(!ObjectUtils.isEmpty(et.getEamlocationtypeid())){
            cn.ibizlab.assetmanagement.core.location.domain.EAMLocationType eamlocationtype=et.getEamlocationtype();
            if(ObjectUtils.isEmpty(eamlocationtype)){
                cn.ibizlab.assetmanagement.core.location.domain.EAMLocationType majorEntity=eamlocationtypeService.get(et.getEamlocationtypeid());
                et.setEamlocationtype(majorEntity);
                eamlocationtype=majorEntity;
            }
            et.setEamlocationtypename(eamlocationtype.getEamlocationtypename());
        }
        //实体关系[DER1N_EAMLOCATION_EAMLOCATION_PEAMLOCATIONID]
        if(!ObjectUtils.isEmpty(et.getPeamlocationid())){
            cn.ibizlab.assetmanagement.core.location.domain.EAMLocation peamlocation=et.getPeamlocation();
            if(ObjectUtils.isEmpty(peamlocation)){
                cn.ibizlab.assetmanagement.core.location.domain.EAMLocation majorEntity=eamlocationService.get(et.getPeamlocationid());
                et.setPeamlocation(majorEntity);
                peamlocation=majorEntity;
            }
            et.setPeamlocationname(peamlocation.getEamlocationname());
        }
    }




    @Override
    public List<JSONObject> select(String sql, Map param){
        return this.baseMapper.selectBySQL(sql,param);
    }

    @Override
    @Transactional
    public boolean execute(String sql , Map param){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql,param);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }

    @Override
    public List<EAMLocation> getEamlocationByIds(List<String> ids) {
         return this.listByIds(ids);
    }

    @Override
    public List<EAMLocation> getEamlocationByEntities(List<EAMLocation> entities) {
        List ids =new ArrayList();
        for(EAMLocation entity : entities){
            Serializable id=entity.getEamlocationid();
            if(!ObjectUtils.isEmpty(id)){
                ids.add(id);
            }
        }
        if(ids.size()>0)
           return this.listByIds(ids);
        else
           return entities;
    }

}



