package cn.ibizlab.assetmanagement.core.location.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import cn.ibizlab.assetmanagement.core.location.domain.EAMLocationType;
import cn.ibizlab.assetmanagement.core.location.filter.EAMLocationTypeSearchContext;
import cn.ibizlab.assetmanagement.core.location.service.IEAMLocationTypeService;

import cn.ibizlab.assetmanagement.util.helper.CachedBeanCopier;
import cn.ibizlab.assetmanagement.util.helper.DEFieldCacheMap;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cn.ibizlab.assetmanagement.core.location.mapper.EAMLocationTypeMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;
import org.springframework.util.StringUtils;

/**
 * 实体[功能位置类型] 服务对象接口实现
 */
@Slf4j
@Service("EAMLocationTypeServiceImpl")
public class EAMLocationTypeServiceImpl extends ServiceImpl<EAMLocationTypeMapper, EAMLocationType> implements IEAMLocationTypeService {

    @Autowired
    @Lazy
    protected cn.ibizlab.assetmanagement.core.location.service.IEAMLocationService eamlocationService;
    @Autowired
    @Lazy
    protected cn.ibizlab.assetmanagement.core.location.service.IEAMLocationStateModelService eamlocationstatemodelService;

    protected int batchSize = 500;

    @Override
    @Transactional
    public boolean create(EAMLocationType et) {
        fillParentData(et);
        if(!this.retBool(this.baseMapper.insert(et)))
            return false;
        CachedBeanCopier.copy(get(et.getEamlocationtypeid()),et);
        return true;
    }

    @Override
    public void createBatch(List<EAMLocationType> list) {
        list.forEach(item->fillParentData(item));
        this.saveBatch(list,batchSize);
    }

    @Override
    @Transactional
    public boolean update(EAMLocationType et) {
        fillParentData(et);
        if(!update(et,(Wrapper) et.getUpdateWrapper(true).eq("eamlocationtypeid",et.getEamlocationtypeid())))
            return false;
        CachedBeanCopier.copy(get(et.getEamlocationtypeid()),et);
        return true;
    }

    @Override
    public void updateBatch(List<EAMLocationType> list) {
        list.forEach(item->fillParentData(item));
        updateBatchById(list,batchSize);
    }

    @Override
    @Transactional
    public boolean remove(String key) {
        boolean result=removeById(key);
        return result ;
    }

    @Override
    public void removeBatch(Collection<String> idList) {
        removeByIds(idList);
    }

    @Override
    @Transactional
    public EAMLocationType get(String key) {
        EAMLocationType et = getById(key);
        if(et==null){
            et=new EAMLocationType();
            et.setEamlocationtypeid(key);
        }
        else{
        }
        return et;
    }

    @Override
    public EAMLocationType getDraft(EAMLocationType et) {
        fillParentData(et);
        return et;
    }

    @Override
    public boolean checkKey(EAMLocationType et) {
        return (!ObjectUtils.isEmpty(et.getEamlocationtypeid()))&&(!Objects.isNull(this.getById(et.getEamlocationtypeid())));
    }
    @Override
    @Transactional
    public boolean save(EAMLocationType et) {
        if(!saveOrUpdate(et))
            return false;
        return true;
    }

    @Override
    @Transactional
    public boolean saveOrUpdate(EAMLocationType et) {
        if (null == et) {
            return false;
        } else {
            return checkKey(et) ? this.update(et) : this.create(et);
        }
    }

    @Override
    public boolean saveBatch(Collection<EAMLocationType> list) {
        list.forEach(item->fillParentData(item));
        saveOrUpdateBatch(list,batchSize);
        return true;
    }

    @Override
    public void saveBatch(List<EAMLocationType> list) {
        list.forEach(item->fillParentData(item));
        saveOrUpdateBatch(list,batchSize);
    }


	@Override
    public List<EAMLocationType> selectByEamlocationstatemodelid(String eamlocationstatemodelid) {
        return baseMapper.selectByEamlocationstatemodelid(eamlocationstatemodelid);
    }

    @Override
    public void removeByEamlocationstatemodelid(String eamlocationstatemodelid) {
        this.remove(new QueryWrapper<EAMLocationType>().eq("eamlocationstatemodelid",eamlocationstatemodelid));
    }


    /**
     * 查询集合 DEFAULT
     */
    @Override
    public Page<EAMLocationType> searchDefault(EAMLocationTypeSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<EAMLocationType> pages=baseMapper.searchDefault(context.getPages(),context,context.getSelectCond());
        return new PageImpl<EAMLocationType>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }



    /**
     * 为当前实体填充父数据（外键值文本、外键值附加数据）
     * @param et
     */
    private void fillParentData(EAMLocationType et){
        //实体关系[DER1N_EAMLOCATIONTYPE_EAMLOCATIONSTATEMODEL_EAMLOCATIONSTATEMODELID]
        if(!ObjectUtils.isEmpty(et.getEamlocationstatemodelid())){
            cn.ibizlab.assetmanagement.core.location.domain.EAMLocationStateModel eamlocationstatemodel=et.getEamlocationstatemodel();
            if(ObjectUtils.isEmpty(eamlocationstatemodel)){
                cn.ibizlab.assetmanagement.core.location.domain.EAMLocationStateModel majorEntity=eamlocationstatemodelService.get(et.getEamlocationstatemodelid());
                et.setEamlocationstatemodel(majorEntity);
                eamlocationstatemodel=majorEntity;
            }
            et.setEamlocationstatemodelname(eamlocationstatemodel.getEamlocationstatemodelname());
        }
    }




    @Override
    public List<JSONObject> select(String sql, Map param){
        return this.baseMapper.selectBySQL(sql,param);
    }

    @Override
    @Transactional
    public boolean execute(String sql , Map param){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql,param);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }

    @Override
    public List<EAMLocationType> getEamlocationtypeByIds(List<String> ids) {
         return this.listByIds(ids);
    }

    @Override
    public List<EAMLocationType> getEamlocationtypeByEntities(List<EAMLocationType> entities) {
        List ids =new ArrayList();
        for(EAMLocationType entity : entities){
            Serializable id=entity.getEamlocationtypeid();
            if(!ObjectUtils.isEmpty(id)){
                ids.add(id);
            }
        }
        if(ids.size()>0)
           return this.listByIds(ids);
        else
           return entities;
    }

}



