package cn.ibizlab.assetmanagement.core.plan.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import cn.ibizlab.assetmanagement.core.plan.domain.EAMPlanCDT;
import cn.ibizlab.assetmanagement.core.plan.filter.EAMPlanCDTSearchContext;
import cn.ibizlab.assetmanagement.core.plan.service.IEAMPlanCDTService;

import cn.ibizlab.assetmanagement.util.helper.CachedBeanCopier;
import cn.ibizlab.assetmanagement.util.helper.DEFieldCacheMap;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cn.ibizlab.assetmanagement.core.plan.mapper.EAMPlanCDTMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;
import org.springframework.util.StringUtils;

/**
 * 实体[维护计划条件] 服务对象接口实现
 */
@Slf4j
@Service("EAMPlanCDTServiceImpl")
public class EAMPlanCDTServiceImpl extends ServiceImpl<EAMPlanCDTMapper, EAMPlanCDT> implements IEAMPlanCDTService {

    @Autowired
    @Lazy
    protected cn.ibizlab.assetmanagement.core.plan.service.IEAMPlanService eamplanService;

    protected int batchSize = 500;

    @Override
    @Transactional
    public boolean create(EAMPlanCDT et) {
        if(!this.retBool(this.baseMapper.insert(et)))
            return false;
        CachedBeanCopier.copy(get(et.getEamplancdtid()),et);
        return true;
    }

    @Override
    public void createBatch(List<EAMPlanCDT> list) {
        this.saveBatch(list,batchSize);
    }

    @Override
    @Transactional
    public boolean update(EAMPlanCDT et) {
        if(!update(et,(Wrapper) et.getUpdateWrapper(true).eq("eamplancdtid",et.getEamplancdtid())))
            return false;
        CachedBeanCopier.copy(get(et.getEamplancdtid()),et);
        return true;
    }

    @Override
    public void updateBatch(List<EAMPlanCDT> list) {
        updateBatchById(list,batchSize);
    }

    @Override
    @Transactional
    public boolean remove(String key) {
        boolean result=removeById(key);
        return result ;
    }

    @Override
    public void removeBatch(Collection<String> idList) {
        removeByIds(idList);
    }

    @Override
    @Transactional
    public EAMPlanCDT get(String key) {
        EAMPlanCDT et = getById(key);
        if(et==null){
            et=new EAMPlanCDT();
            et.setEamplancdtid(key);
        }
        else{
        }
        return et;
    }

    @Override
    public EAMPlanCDT getDraft(EAMPlanCDT et) {
        return et;
    }

    @Override
    public boolean checkKey(EAMPlanCDT et) {
        return (!ObjectUtils.isEmpty(et.getEamplancdtid()))&&(!Objects.isNull(this.getById(et.getEamplancdtid())));
    }
    @Override
    @Transactional
    public boolean save(EAMPlanCDT et) {
        if(!saveOrUpdate(et))
            return false;
        return true;
    }

    @Override
    @Transactional
    public boolean saveOrUpdate(EAMPlanCDT et) {
        if (null == et) {
            return false;
        } else {
            return checkKey(et) ? this.update(et) : this.create(et);
        }
    }

    @Override
    public boolean saveBatch(Collection<EAMPlanCDT> list) {
        saveOrUpdateBatch(list,batchSize);
        return true;
    }

    @Override
    public void saveBatch(List<EAMPlanCDT> list) {
        saveOrUpdateBatch(list,batchSize);
    }


	@Override
    public List<EAMPlanCDT> selectByEamplanid(String eamplanid) {
        return baseMapper.selectByEamplanid(eamplanid);
    }

    @Override
    public void removeByEamplanid(String eamplanid) {
        this.remove(new QueryWrapper<EAMPlanCDT>().eq("eamplanid",eamplanid));
    }


    /**
     * 查询集合 DEFAULT
     */
    @Override
    public Page<EAMPlanCDT> searchDefault(EAMPlanCDTSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<EAMPlanCDT> pages=baseMapper.searchDefault(context.getPages(),context,context.getSelectCond());
        return new PageImpl<EAMPlanCDT>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }







    @Override
    public List<JSONObject> select(String sql, Map param){
        return this.baseMapper.selectBySQL(sql,param);
    }

    @Override
    @Transactional
    public boolean execute(String sql , Map param){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql,param);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }

    @Override
    public List<EAMPlanCDT> getEamplancdtByIds(List<String> ids) {
         return this.listByIds(ids);
    }

    @Override
    public List<EAMPlanCDT> getEamplancdtByEntities(List<EAMPlanCDT> entities) {
        List ids =new ArrayList();
        for(EAMPlanCDT entity : entities){
            Serializable id=entity.getEamplancdtid();
            if(!ObjectUtils.isEmpty(id)){
                ids.add(id);
            }
        }
        if(ids.size()>0)
           return this.listByIds(ids);
        else
           return entities;
    }

}



