package cn.ibizlab.assetmanagement.core.plan.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import cn.ibizlab.assetmanagement.core.plan.domain.EAMPlanLine;
import cn.ibizlab.assetmanagement.core.plan.filter.EAMPlanLineSearchContext;
import cn.ibizlab.assetmanagement.core.plan.service.IEAMPlanLineService;

import cn.ibizlab.assetmanagement.util.helper.CachedBeanCopier;
import cn.ibizlab.assetmanagement.util.helper.DEFieldCacheMap;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cn.ibizlab.assetmanagement.core.plan.mapper.EAMPlanLineMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;
import org.springframework.util.StringUtils;

/**
 * 实体[计划步骤] 服务对象接口实现
 */
@Slf4j
@Service("EAMPlanLineServiceImpl")
public class EAMPlanLineServiceImpl extends ServiceImpl<EAMPlanLineMapper, EAMPlanLine> implements IEAMPlanLineService {

    @Autowired
    @Lazy
    protected cn.ibizlab.assetmanagement.core.asset.service.IEAMAssetService eamassetService;
    @Autowired
    @Lazy
    protected cn.ibizlab.assetmanagement.core.location.service.IEAMLocationService eamlocationService;
    @Autowired
    @Lazy
    protected cn.ibizlab.assetmanagement.core.workorder.service.IEAMMonitorPointService eammonitorpointService;
    @Autowired
    @Lazy
    protected cn.ibizlab.assetmanagement.core.plan.service.IEAMPlanService eamplanService;

    protected int batchSize = 500;

    @Override
    @Transactional
    public boolean create(EAMPlanLine et) {
        fillParentData(et);
        if(!this.retBool(this.baseMapper.insert(et)))
            return false;
        CachedBeanCopier.copy(get(et.getEamplanlineid()),et);
        return true;
    }

    @Override
    public void createBatch(List<EAMPlanLine> list) {
        list.forEach(item->fillParentData(item));
        this.saveBatch(list,batchSize);
    }

    @Override
    @Transactional
    public boolean update(EAMPlanLine et) {
        fillParentData(et);
        if(!update(et,(Wrapper) et.getUpdateWrapper(true).eq("eamplanlineid",et.getEamplanlineid())))
            return false;
        CachedBeanCopier.copy(get(et.getEamplanlineid()),et);
        return true;
    }

    @Override
    public void updateBatch(List<EAMPlanLine> list) {
        list.forEach(item->fillParentData(item));
        updateBatchById(list,batchSize);
    }

    @Override
    @Transactional
    public boolean remove(String key) {
        boolean result=removeById(key);
        return result ;
    }

    @Override
    public void removeBatch(Collection<String> idList) {
        removeByIds(idList);
    }

    @Override
    @Transactional
    public EAMPlanLine get(String key) {
        EAMPlanLine et = getById(key);
        if(et==null){
            et=new EAMPlanLine();
            et.setEamplanlineid(key);
        }
        else{
        }
        return et;
    }

    @Override
    public EAMPlanLine getDraft(EAMPlanLine et) {
        fillParentData(et);
        return et;
    }

    @Override
    public boolean checkKey(EAMPlanLine et) {
        return (!ObjectUtils.isEmpty(et.getEamplanlineid()))&&(!Objects.isNull(this.getById(et.getEamplanlineid())));
    }
    @Override
    @Transactional
    public boolean save(EAMPlanLine et) {
        if(!saveOrUpdate(et))
            return false;
        return true;
    }

    @Override
    @Transactional
    public boolean saveOrUpdate(EAMPlanLine et) {
        if (null == et) {
            return false;
        } else {
            return checkKey(et) ? this.update(et) : this.create(et);
        }
    }

    @Override
    public boolean saveBatch(Collection<EAMPlanLine> list) {
        list.forEach(item->fillParentData(item));
        saveOrUpdateBatch(list,batchSize);
        return true;
    }

    @Override
    public void saveBatch(List<EAMPlanLine> list) {
        list.forEach(item->fillParentData(item));
        saveOrUpdateBatch(list,batchSize);
    }


	@Override
    public List<EAMPlanLine> selectByAssetid(String assetid) {
        return baseMapper.selectByAssetid(assetid);
    }

    @Override
    public void removeByAssetid(String assetid) {
        this.remove(new QueryWrapper<EAMPlanLine>().eq("assetid",assetid));
    }

	@Override
    public List<EAMPlanLine> selectByEamlocationid(String eamlocationid) {
        return baseMapper.selectByEamlocationid(eamlocationid);
    }

    @Override
    public void removeByEamlocationid(String eamlocationid) {
        this.remove(new QueryWrapper<EAMPlanLine>().eq("eamlocationid",eamlocationid));
    }

	@Override
    public List<EAMPlanLine> selectByEammonitorpointid(String eammonitorpointid) {
        return baseMapper.selectByEammonitorpointid(eammonitorpointid);
    }

    @Override
    public void removeByEammonitorpointid(String eammonitorpointid) {
        this.remove(new QueryWrapper<EAMPlanLine>().eq("eammonitorpointid",eammonitorpointid));
    }

	@Override
    public List<EAMPlanLine> selectByEamplanid(String eamplanid) {
        return baseMapper.selectByEamplanid(eamplanid);
    }

    @Override
    public void removeByEamplanid(String eamplanid) {
        this.remove(new QueryWrapper<EAMPlanLine>().eq("eamplanid",eamplanid));
    }


    /**
     * 查询集合 DEFAULT
     */
    @Override
    public Page<EAMPlanLine> searchDefault(EAMPlanLineSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<EAMPlanLine> pages=baseMapper.searchDefault(context.getPages(),context,context.getSelectCond());
        return new PageImpl<EAMPlanLine>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }



    /**
     * 为当前实体填充父数据（外键值文本、外键值附加数据）
     * @param et
     */
    private void fillParentData(EAMPlanLine et){
        //实体关系[DER1N_EAMPLANLINE_EAMASSET_ASSETID]
        if(!ObjectUtils.isEmpty(et.getAssetid())){
            cn.ibizlab.assetmanagement.core.asset.domain.EAMAsset asset=et.getAsset();
            if(ObjectUtils.isEmpty(asset)){
                cn.ibizlab.assetmanagement.core.asset.domain.EAMAsset majorEntity=eamassetService.get(et.getAssetid());
                et.setAsset(majorEntity);
                asset=majorEntity;
            }
            et.setAssetname(asset.getAssetname());
        }
        //实体关系[DER1N_EAMPLANLINE_EAMLOCATION_EAMLOCATIONID]
        if(!ObjectUtils.isEmpty(et.getEamlocationid())){
            cn.ibizlab.assetmanagement.core.location.domain.EAMLocation eamlocation=et.getEamlocation();
            if(ObjectUtils.isEmpty(eamlocation)){
                cn.ibizlab.assetmanagement.core.location.domain.EAMLocation majorEntity=eamlocationService.get(et.getEamlocationid());
                et.setEamlocation(majorEntity);
                eamlocation=majorEntity;
            }
            et.setEamlocationname(eamlocation.getEamlocationname());
        }
        //实体关系[DER1N_EAMPLANLINE_EAMMONITORPOINT_EAMMONITORPOINTID]
        if(!ObjectUtils.isEmpty(et.getEammonitorpointid())){
            cn.ibizlab.assetmanagement.core.workorder.domain.EAMMonitorPoint eammonitorpoint=et.getEammonitorpoint();
            if(ObjectUtils.isEmpty(eammonitorpoint)){
                cn.ibizlab.assetmanagement.core.workorder.domain.EAMMonitorPoint majorEntity=eammonitorpointService.get(et.getEammonitorpointid());
                et.setEammonitorpoint(majorEntity);
                eammonitorpoint=majorEntity;
            }
            et.setEammonitorpointname(eammonitorpoint.getEammonitorpointname());
        }
        //实体关系[DER1N_EAMPLANLINE_EAMPLAN_EAMPLANID]
        if(!ObjectUtils.isEmpty(et.getEamplanid())){
            cn.ibizlab.assetmanagement.core.plan.domain.EAMPlan eamplan=et.getEamplan();
            if(ObjectUtils.isEmpty(eamplan)){
                cn.ibizlab.assetmanagement.core.plan.domain.EAMPlan majorEntity=eamplanService.get(et.getEamplanid());
                et.setEamplan(majorEntity);
                eamplan=majorEntity;
            }
            et.setEamplanname(eamplan.getEamplanname());
        }
    }




    @Override
    public List<JSONObject> select(String sql, Map param){
        return this.baseMapper.selectBySQL(sql,param);
    }

    @Override
    @Transactional
    public boolean execute(String sql , Map param){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql,param);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }

    @Override
    public List<EAMPlanLine> getEamplanlineByIds(List<String> ids) {
         return this.listByIds(ids);
    }

    @Override
    public List<EAMPlanLine> getEamplanlineByEntities(List<EAMPlanLine> entities) {
        List ids =new ArrayList();
        for(EAMPlanLine entity : entities){
            Serializable id=entity.getEamplanlineid();
            if(!ObjectUtils.isEmpty(id)){
                ids.add(id);
            }
        }
        if(ids.size()>0)
           return this.listByIds(ids);
        else
           return entities;
    }

}



