package cn.ibizlab.assetmanagement.core.plan.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import cn.ibizlab.assetmanagement.core.plan.domain.EAMPlan;
import cn.ibizlab.assetmanagement.core.plan.filter.EAMPlanSearchContext;
import cn.ibizlab.assetmanagement.core.plan.service.IEAMPlanService;

import cn.ibizlab.assetmanagement.util.helper.CachedBeanCopier;
import cn.ibizlab.assetmanagement.util.helper.DEFieldCacheMap;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cn.ibizlab.assetmanagement.core.plan.mapper.EAMPlanMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;
import org.springframework.util.StringUtils;

/**
 * 实体[维护计划] 服务对象接口实现
 */
@Slf4j
@Service("EAMPlanServiceImpl")
public class EAMPlanServiceImpl extends ServiceImpl<EAMPlanMapper, EAMPlan> implements IEAMPlanService {

    @Autowired
    @Lazy
    protected cn.ibizlab.assetmanagement.core.plan.service.IEAMPlanCDTService eamplancdtService;
    @Autowired
    @Lazy
    protected cn.ibizlab.assetmanagement.core.plan.service.IEAMPlanLineService eamplanlineService;
    @Autowired
    @Lazy
    protected cn.ibizlab.assetmanagement.core.plan.service.IEAMPlanScheduleService eamplanscheduleService;
    @Autowired
    @Lazy
    protected cn.ibizlab.assetmanagement.core.asset.service.IEAMAssetService eamassetService;
    @Autowired
    @Lazy
    protected cn.ibizlab.assetmanagement.core.location.service.IEAMLocationService eamlocationService;
    @Autowired
    @Lazy
    protected cn.ibizlab.assetmanagement.core.workorder.service.IEAMMonitorPointService eammonitorpointService;

    protected int batchSize = 500;

    @Override
    @Transactional
    public boolean create(EAMPlan et) {
        fillParentData(et);
        createIndexMajorEntityData(et);
        if(!this.retBool(this.baseMapper.insert(et)))
            return false;
        CachedBeanCopier.copy(get(et.getEamplanid()),et);
        return true;
    }

    @Override
    public void createBatch(List<EAMPlan> list) {
        list.forEach(item->fillParentData(item));
        this.saveBatch(list,batchSize);
    }

    @Override
    @Transactional
    public boolean update(EAMPlan et) {
        fillParentData(et);
        wooriginService.update(eamplanInheritMapping.toWoorigin(et));
        if(!update(et,(Wrapper) et.getUpdateWrapper(true).eq("eamplanid",et.getEamplanid())))
            return false;
        CachedBeanCopier.copy(get(et.getEamplanid()),et);
        return true;
    }

    @Override
    public void updateBatch(List<EAMPlan> list) {
        list.forEach(item->fillParentData(item));
        updateBatchById(list,batchSize);
    }

    @Override
    @Transactional
    public boolean remove(String key) {
        wooriginService.remove(key);
        boolean result=removeById(key);
        return result ;
    }

    @Override
    public void removeBatch(Collection<String> idList) {
        removeByIds(idList);
    }

    @Override
    @Transactional
    public EAMPlan get(String key) {
        EAMPlan et = getById(key);
        if(et==null){
            et=new EAMPlan();
            et.setEamplanid(key);
        }
        else{
        }
        return et;
    }

    @Override
    public EAMPlan getDraft(EAMPlan et) {
        fillParentData(et);
        return et;
    }

    @Override
    public boolean checkKey(EAMPlan et) {
        return (!ObjectUtils.isEmpty(et.getEamplanid()))&&(!Objects.isNull(this.getById(et.getEamplanid())));
    }
    @Override
    @Transactional
    public boolean save(EAMPlan et) {
        if(!saveOrUpdate(et))
            return false;
        return true;
    }

    @Override
    @Transactional
    public boolean saveOrUpdate(EAMPlan et) {
        if (null == et) {
            return false;
        } else {
            return checkKey(et) ? this.update(et) : this.create(et);
        }
    }

    @Override
    public boolean saveBatch(Collection<EAMPlan> list) {
        list.forEach(item->fillParentData(item));
        saveOrUpdateBatch(list,batchSize);
        return true;
    }

    @Override
    public void saveBatch(List<EAMPlan> list) {
        list.forEach(item->fillParentData(item));
        saveOrUpdateBatch(list,batchSize);
    }


	@Override
    public List<EAMPlan> selectByAssetid(String assetid) {
        return baseMapper.selectByAssetid(assetid);
    }

    @Override
    public void removeByAssetid(String assetid) {
        this.remove(new QueryWrapper<EAMPlan>().eq("assetid",assetid));
    }

	@Override
    public List<EAMPlan> selectByEamlocationid(String eamlocationid) {
        return baseMapper.selectByEamlocationid(eamlocationid);
    }

    @Override
    public void removeByEamlocationid(String eamlocationid) {
        this.remove(new QueryWrapper<EAMPlan>().eq("eamlocationid",eamlocationid));
    }

	@Override
    public List<EAMPlan> selectByEammonitorpointid(String eammonitorpointid) {
        return baseMapper.selectByEammonitorpointid(eammonitorpointid);
    }

    @Override
    public void removeByEammonitorpointid(String eammonitorpointid) {
        this.remove(new QueryWrapper<EAMPlan>().eq("eammonitorpointid",eammonitorpointid));
    }


    /**
     * 查询集合 DEFAULT
     */
    @Override
    public Page<EAMPlan> searchDefault(EAMPlanSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<EAMPlan> pages=baseMapper.searchDefault(context.getPages(),context,context.getSelectCond());
        return new PageImpl<EAMPlan>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }



    /**
     * 为当前实体填充父数据（外键值文本、外键值附加数据）
     * @param et
     */
    private void fillParentData(EAMPlan et){
        //实体关系[DER1N_EAMPLAN_EAMASSET_ASSETID]
        if(!ObjectUtils.isEmpty(et.getAssetid())){
            cn.ibizlab.assetmanagement.core.asset.domain.EAMAsset asset=et.getAsset();
            if(ObjectUtils.isEmpty(asset)){
                cn.ibizlab.assetmanagement.core.asset.domain.EAMAsset majorEntity=eamassetService.get(et.getAssetid());
                et.setAsset(majorEntity);
                asset=majorEntity;
            }
            et.setAssetname(asset.getAssetname());
        }
        //实体关系[DER1N_EAMPLAN_EAMLOCATION_EAMLOCATIONID]
        if(!ObjectUtils.isEmpty(et.getEamlocationid())){
            cn.ibizlab.assetmanagement.core.location.domain.EAMLocation eamlocation=et.getEamlocation();
            if(ObjectUtils.isEmpty(eamlocation)){
                cn.ibizlab.assetmanagement.core.location.domain.EAMLocation majorEntity=eamlocationService.get(et.getEamlocationid());
                et.setEamlocation(majorEntity);
                eamlocation=majorEntity;
            }
            et.setEamlocationname(eamlocation.getEamlocationname());
        }
        //实体关系[DER1N_EAMPLAN_EAMMONITORPOINT_EAMMONITORPOINTID]
        if(!ObjectUtils.isEmpty(et.getEammonitorpointid())){
            cn.ibizlab.assetmanagement.core.workorder.domain.EAMMonitorPoint eammonitorpoint=et.getEammonitorpoint();
            if(ObjectUtils.isEmpty(eammonitorpoint)){
                cn.ibizlab.assetmanagement.core.workorder.domain.EAMMonitorPoint majorEntity=eammonitorpointService.get(et.getEammonitorpointid());
                et.setEammonitorpoint(majorEntity);
                eammonitorpoint=majorEntity;
            }
            et.setEammonitorpointname(eammonitorpoint.getEammonitorpointname());
        }
    }



    @Autowired
    cn.ibizlab.assetmanagement.core.plan.mapping.EAMPlanInheritMapping eamplanInheritMapping;
    @Autowired
    @Lazy
    protected cn.ibizlab.assetmanagement.core.workorder.service.IWOOriginService wooriginService;

    /**
     * 创建索引主实体数据
     * @param et
     */
    private void createIndexMajorEntityData(EAMPlan et){
        if(ObjectUtils.isEmpty(et.getEamplanid()))
            et.setEamplanid((String)et.getDefaultKey(true));
        cn.ibizlab.assetmanagement.core.workorder.domain.WOOrigin woorigin =eamplanInheritMapping.toWoorigin(et);
        woorigin.set("woorigintype","PLAN");
        wooriginService.create(woorigin);
    }

    @Override
    public List<JSONObject> select(String sql, Map param){
        return this.baseMapper.selectBySQL(sql,param);
    }

    @Override
    @Transactional
    public boolean execute(String sql , Map param){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql,param);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }

    @Override
    public List<EAMPlan> getEamplanByIds(List<String> ids) {
         return this.listByIds(ids);
    }

    @Override
    public List<EAMPlan> getEamplanByEntities(List<EAMPlan> entities) {
        List ids =new ArrayList();
        for(EAMPlan entity : entities){
            Serializable id=entity.getEamplanid();
            if(!ObjectUtils.isEmpty(id)){
                ids.add(id);
            }
        }
        if(ids.size()>0)
           return this.listByIds(ids);
        else
           return entities;
    }

}



