package cn.ibizlab.assetmanagement.core.workorder.domain;

import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.math.BigInteger;
import java.util.HashMap;
import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import com.alibaba.fastjson.annotation.JSONField;
import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.annotation.JsonFormat;
import org.springframework.util.ObjectUtils;
import org.springframework.util.DigestUtils;
import cn.ibizlab.assetmanagement.util.domain.EntityBase;
import cn.ibizlab.assetmanagement.util.annotation.DEField;
import cn.ibizlab.assetmanagement.util.enums.DEPredefinedFieldType;
import cn.ibizlab.assetmanagement.util.enums.DEFieldDefaultValueType;
import java.io.Serializable;
import lombok.*;
import org.springframework.data.annotation.Transient;
import cn.ibizlab.assetmanagement.util.annotation.Audit;


import com.fasterxml.jackson.annotation.JsonIgnoreProperties;
import com.baomidou.mybatisplus.annotation.*;
import cn.ibizlab.assetmanagement.util.domain.EntityMP;

/**
 * 实体[工单]
 */
@Getter
@Setter
@NoArgsConstructor
@JsonIgnoreProperties(value = "handler")
@TableName(value = "WORKORDER",resultMap = "EAMWorkOrderResultMap")
public class EAMWorkOrder extends EntityMP implements Serializable {

    private static final long serialVersionUID = 1L;

    /**
     * 工单标识
     */
    @DEField(isKeyField=true)
    @TableId(value= "workorderid",type=IdType.ASSIGN_UUID)
    @JSONField(name = "workorderid")
    @JsonProperty("workorderid")
    private String workorderid;
    /**
     * 工单名称
     */
    @TableField(value = "workordername")
    @JSONField(name = "workordername")
    @JsonProperty("workordername")
    private String workordername;
    /**
     * 建立时间
     */
    @DEField(preType = DEPredefinedFieldType.CREATEDATE)
    @TableField(value = "createdate" , fill = FieldFill.INSERT)
    @JsonFormat(pattern="yyyy-MM-dd HH:mm:ss", locale = "zh" , timezone="GMT+8")
    @JSONField(name = "createdate" , format="yyyy-MM-dd HH:mm:ss")
    @JsonProperty("createdate")
    private Timestamp createdate;
    /**
     * 建立人
     */
    @DEField(preType = DEPredefinedFieldType.CREATEMAN)
    @TableField(value = "createman" , fill = FieldFill.INSERT)
    @JSONField(name = "createman")
    @JsonProperty("createman")
    private String createman;
    /**
     * 更新时间
     */
    @DEField(preType = DEPredefinedFieldType.UPDATEDATE)
    @TableField(value = "updatedate")
    @JsonFormat(pattern="yyyy-MM-dd HH:mm:ss", locale = "zh" , timezone="GMT+8")
    @JSONField(name = "updatedate" , format="yyyy-MM-dd HH:mm:ss")
    @JsonProperty("updatedate")
    private Timestamp updatedate;
    /**
     * 更新人
     */
    @DEField(preType = DEPredefinedFieldType.UPDATEMAN)
    @TableField(value = "updateman")
    @JSONField(name = "updateman")
    @JsonProperty("updateman")
    private String updateman;
    /**
     * 工单类型标识
     */
    @TableField(value = "eamwotypeid")
    @JSONField(name = "eamwotypeid")
    @JsonProperty("eamwotypeid")
    private String eamwotypeid;
    /**
     * 资产标识
     */
    @TableField(value = "assetid")
    @JSONField(name = "assetid")
    @JsonProperty("assetid")
    private String assetid;
    /**
     * 功能位置标识
     */
    @TableField(value = "eamlocationid")
    @JSONField(name = "eamlocationid")
    @JsonProperty("eamlocationid")
    private String eamlocationid;
    /**
     * 上级工单标识
     */
    @TableField(value = "pworkorderid")
    @JSONField(name = "pworkorderid")
    @JsonProperty("pworkorderid")
    private String pworkorderid;
    /**
     * 工单来源标识
     */
    @DEField(name = "eamwooriginid")
    @TableField(value = "eamwooriginid")
    @JSONField(name = "wooriginid")
    @JsonProperty("wooriginid")
    private String wooriginid;
    /**
     * 工单编号
     */
    @TableField(value = "wonumber")
    @JSONField(name = "wonumber")
    @JsonProperty("wonumber")
    private String wonumber;
    /**
     * 制定日期
     */
    @TableField(value = "mdate")
    @JsonFormat(pattern="yyyy-MM-dd HH:mm:ss", locale = "zh" , timezone="GMT+8")
    @JSONField(name = "mdate" , format="yyyy-MM-dd HH:mm:ss")
    @JsonProperty("mdate")
    private Timestamp mdate;
    /**
     * 执行日期
     */
    @TableField(value = "wodate")
    @JsonFormat(pattern="yyyy-MM-dd HH:mm:ss", locale = "zh" , timezone="GMT+8")
    @JSONField(name = "wodate" , format="yyyy-MM-dd HH:mm:ss")
    @JsonProperty("wodate")
    private Timestamp wodate;
    /**
     * 实际开始时间
     */
    @TableField(value = "begintime")
    @JsonFormat(pattern="yyyy-MM-dd HH:mm:ss", locale = "zh" , timezone="GMT+8")
    @JSONField(name = "begintime" , format="yyyy-MM-dd HH:mm:ss")
    @JsonProperty("begintime")
    private Timestamp begintime;
    /**
     * 实际结束时间
     */
    @TableField(value = "endtime")
    @JsonFormat(pattern="yyyy-MM-dd HH:mm:ss", locale = "zh" , timezone="GMT+8")
    @JSONField(name = "endtime" , format="yyyy-MM-dd HH:mm:ss")
    @JsonProperty("endtime")
    private Timestamp endtime;
    /**
     * 服务等级标识
     */
    @TableField(value = "eamservicelevelid")
    @JSONField(name = "eamservicelevelid")
    @JsonProperty("eamservicelevelid")
    private String eamservicelevelid;
    /**
     * 工时
     */
    @TableField(value = "worklength")
    @JSONField(name = "worklength")
    @JsonProperty("worklength")
    private Double worklength;
    /**
     * 资产
     */
    @TableField(exist = false)
    @JSONField(name = "assetname")
    @JsonProperty("assetname")
    private String assetname;
    /**
     * 功能位置
     */
    @TableField(exist = false)
    @JSONField(name = "eamlocationname")
    @JsonProperty("eamlocationname")
    private String eamlocationname;
    /**
     * 服务等级
     */
    @TableField(exist = false)
    @JSONField(name = "eamservicelevelname")
    @JsonProperty("eamservicelevelname")
    private String eamservicelevelname;
    /**
     * 工单来源
     */
    @TableField(exist = false)
    @JSONField(name = "wooriginname")
    @JsonProperty("wooriginname")
    private String wooriginname;
    /**
     * 工单类型
     */
    @TableField(exist = false)
    @JSONField(name = "eamwotypename")
    @JsonProperty("eamwotypename")
    private String eamwotypename;
    /**
     * 上级工单
     */
    @TableField(exist = false)
    @JSONField(name = "pworkordername")
    @JsonProperty("pworkordername")
    private String pworkordername;
    /**
     * 描述
     */
    @TableField(value = "description")
    @JSONField(name = "description")
    @JsonProperty("description")
    private String description;

    /**
     * 
     */
    @JsonIgnore
    @JSONField(serialize = false)
    @TableField(exist = false)
    private cn.ibizlab.assetmanagement.core.asset.domain.EAMAsset asset;

    /**
     * 
     */
    @JsonIgnore
    @JSONField(serialize = false)
    @TableField(exist = false)
    private cn.ibizlab.assetmanagement.core.location.domain.EAMLocation eamlocation;

    /**
     * 
     */
    @JsonIgnore
    @JSONField(serialize = false)
    @TableField(exist = false)
    private cn.ibizlab.assetmanagement.core.workorder.domain.EAMServiceLevel eamservicelevel;

    /**
     * 
     */
    @JsonIgnore
    @JSONField(serialize = false)
    @TableField(exist = false)
    private cn.ibizlab.assetmanagement.core.workorder.domain.WOOrigin eamwoorigin;

    /**
     * 
     */
    @JsonIgnore
    @JSONField(serialize = false)
    @TableField(exist = false)
    private cn.ibizlab.assetmanagement.core.workorder.domain.EAMWorkOrder pworkorder;

    /**
     * 
     */
    @JsonIgnore
    @JSONField(serialize = false)
    @TableField(exist = false)
    private cn.ibizlab.assetmanagement.core.workorder.domain.EAMWOType eamwotype;



    /**
     * 设置 [工单名称]
     */
    public void setWorkordername(String workordername){
        this.workordername = workordername ;
        this.modify("workordername",workordername);
    }

    /**
     * 设置 [工单类型标识]
     */
    public void setEamwotypeid(String eamwotypeid){
        this.eamwotypeid = eamwotypeid ;
        this.modify("eamwotypeid",eamwotypeid);
    }

    /**
     * 设置 [资产标识]
     */
    public void setAssetid(String assetid){
        this.assetid = assetid ;
        this.modify("assetid",assetid);
    }

    /**
     * 设置 [功能位置标识]
     */
    public void setEamlocationid(String eamlocationid){
        this.eamlocationid = eamlocationid ;
        this.modify("eamlocationid",eamlocationid);
    }

    /**
     * 设置 [上级工单标识]
     */
    public void setPworkorderid(String pworkorderid){
        this.pworkorderid = pworkorderid ;
        this.modify("pworkorderid",pworkorderid);
    }

    /**
     * 设置 [工单来源标识]
     */
    public void setWooriginid(String wooriginid){
        this.wooriginid = wooriginid ;
        this.modify("eamwooriginid",wooriginid);
    }

    /**
     * 设置 [工单编号]
     */
    public void setWonumber(String wonumber){
        this.wonumber = wonumber ;
        this.modify("wonumber",wonumber);
    }

    /**
     * 设置 [制定日期]
     */
    public void setMdate(Timestamp mdate){
        this.mdate = mdate ;
        this.modify("mdate",mdate);
    }

    /**
     * 格式化日期 [制定日期]
     */
    public String formatMdate(){
        if (this.mdate == null) {
            return null;
        }
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        return sdf.format(mdate);
    }
    /**
     * 设置 [执行日期]
     */
    public void setWodate(Timestamp wodate){
        this.wodate = wodate ;
        this.modify("wodate",wodate);
    }

    /**
     * 格式化日期 [执行日期]
     */
    public String formatWodate(){
        if (this.wodate == null) {
            return null;
        }
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        return sdf.format(wodate);
    }
    /**
     * 设置 [实际开始时间]
     */
    public void setBegintime(Timestamp begintime){
        this.begintime = begintime ;
        this.modify("begintime",begintime);
    }

    /**
     * 格式化日期 [实际开始时间]
     */
    public String formatBegintime(){
        if (this.begintime == null) {
            return null;
        }
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        return sdf.format(begintime);
    }
    /**
     * 设置 [实际结束时间]
     */
    public void setEndtime(Timestamp endtime){
        this.endtime = endtime ;
        this.modify("endtime",endtime);
    }

    /**
     * 格式化日期 [实际结束时间]
     */
    public String formatEndtime(){
        if (this.endtime == null) {
            return null;
        }
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        return sdf.format(endtime);
    }
    /**
     * 设置 [服务等级标识]
     */
    public void setEamservicelevelid(String eamservicelevelid){
        this.eamservicelevelid = eamservicelevelid ;
        this.modify("eamservicelevelid",eamservicelevelid);
    }

    /**
     * 设置 [工时]
     */
    public void setWorklength(Double worklength){
        this.worklength = worklength ;
        this.modify("worklength",worklength);
    }

    /**
     * 设置 [描述]
     */
    public void setDescription(String description){
        this.description = description ;
        this.modify("description",description);
    }


}


