package cn.ibizlab.assetmanagement.core.workorder.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import cn.ibizlab.assetmanagement.core.workorder.domain.EAMWOState;
import cn.ibizlab.assetmanagement.core.workorder.filter.EAMWOStateSearchContext;
import cn.ibizlab.assetmanagement.core.workorder.service.IEAMWOStateService;

import cn.ibizlab.assetmanagement.util.helper.CachedBeanCopier;
import cn.ibizlab.assetmanagement.util.helper.DEFieldCacheMap;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cn.ibizlab.assetmanagement.core.workorder.mapper.EAMWOStateMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;
import org.springframework.util.StringUtils;

/**
 * 实体[工单状态] 服务对象接口实现
 */
@Slf4j
@Service("EAMWOStateServiceImpl")
public class EAMWOStateServiceImpl extends ServiceImpl<EAMWOStateMapper, EAMWOState> implements IEAMWOStateService {

    @Autowired
    @Lazy
    protected cn.ibizlab.assetmanagement.core.workorder.service.IEAMWOStateModelLineService eamwostatemodellineService;

    protected int batchSize = 500;

    @Override
    @Transactional
    public boolean create(EAMWOState et) {
        if(!this.retBool(this.baseMapper.insert(et)))
            return false;
        CachedBeanCopier.copy(get(et.getEamwostateid()),et);
        return true;
    }

    @Override
    public void createBatch(List<EAMWOState> list) {
        this.saveBatch(list,batchSize);
    }

    @Override
    @Transactional
    public boolean update(EAMWOState et) {
        if(!update(et,(Wrapper) et.getUpdateWrapper(true).eq("eamwostateid",et.getEamwostateid())))
            return false;
        CachedBeanCopier.copy(get(et.getEamwostateid()),et);
        return true;
    }

    @Override
    public void updateBatch(List<EAMWOState> list) {
        updateBatchById(list,batchSize);
    }

    @Override
    @Transactional
    public boolean remove(String key) {
        boolean result=removeById(key);
        return result ;
    }

    @Override
    public void removeBatch(Collection<String> idList) {
        removeByIds(idList);
    }

    @Override
    @Transactional
    public EAMWOState get(String key) {
        EAMWOState et = getById(key);
        if(et==null){
            et=new EAMWOState();
            et.setEamwostateid(key);
        }
        else{
        }
        return et;
    }

    @Override
    public EAMWOState getDraft(EAMWOState et) {
        return et;
    }

    @Override
    public boolean checkKey(EAMWOState et) {
        return (!ObjectUtils.isEmpty(et.getEamwostateid()))&&(!Objects.isNull(this.getById(et.getEamwostateid())));
    }
    @Override
    @Transactional
    public boolean save(EAMWOState et) {
        if(!saveOrUpdate(et))
            return false;
        return true;
    }

    @Override
    @Transactional
    public boolean saveOrUpdate(EAMWOState et) {
        if (null == et) {
            return false;
        } else {
            return checkKey(et) ? this.update(et) : this.create(et);
        }
    }

    @Override
    public boolean saveBatch(Collection<EAMWOState> list) {
        saveOrUpdateBatch(list,batchSize);
        return true;
    }

    @Override
    public void saveBatch(List<EAMWOState> list) {
        saveOrUpdateBatch(list,batchSize);
    }



    /**
     * 查询集合 DEFAULT
     */
    @Override
    public Page<EAMWOState> searchDefault(EAMWOStateSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<EAMWOState> pages=baseMapper.searchDefault(context.getPages(),context,context.getSelectCond());
        return new PageImpl<EAMWOState>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }







    @Override
    public List<JSONObject> select(String sql, Map param){
        return this.baseMapper.selectBySQL(sql,param);
    }

    @Override
    @Transactional
    public boolean execute(String sql , Map param){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql,param);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }

    @Override
    public List<EAMWOState> getEamwostateByIds(List<String> ids) {
         return this.listByIds(ids);
    }

    @Override
    public List<EAMWOState> getEamwostateByEntities(List<EAMWOState> entities) {
        List ids =new ArrayList();
        for(EAMWOState entity : entities){
            Serializable id=entity.getEamwostateid();
            if(!ObjectUtils.isEmpty(id)){
                ids.add(id);
            }
        }
        if(ids.size()>0)
           return this.listByIds(ids);
        else
           return entities;
    }

}



