package cn.ibizlab.assetmanagement.core.workorder.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import cn.ibizlab.assetmanagement.core.workorder.domain.EAMWOType;
import cn.ibizlab.assetmanagement.core.workorder.filter.EAMWOTypeSearchContext;
import cn.ibizlab.assetmanagement.core.workorder.service.IEAMWOTypeService;

import cn.ibizlab.assetmanagement.util.helper.CachedBeanCopier;
import cn.ibizlab.assetmanagement.util.helper.DEFieldCacheMap;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cn.ibizlab.assetmanagement.core.workorder.mapper.EAMWOTypeMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;
import org.springframework.util.StringUtils;

/**
 * 实体[工单类型] 服务对象接口实现
 */
@Slf4j
@Service("EAMWOTypeServiceImpl")
public class EAMWOTypeServiceImpl extends ServiceImpl<EAMWOTypeMapper, EAMWOType> implements IEAMWOTypeService {

    @Autowired
    @Lazy
    protected cn.ibizlab.assetmanagement.core.workorder.service.IEAMWorkOrderService eamworkorderService;
    @Autowired
    @Lazy
    protected cn.ibizlab.assetmanagement.core.workorder.service.IEAMWOStateModelService eamwostatemodelService;

    protected int batchSize = 500;

    @Override
    @Transactional
    public boolean create(EAMWOType et) {
        if(!this.retBool(this.baseMapper.insert(et)))
            return false;
        CachedBeanCopier.copy(get(et.getEamwotypeid()),et);
        return true;
    }

    @Override
    public void createBatch(List<EAMWOType> list) {
        this.saveBatch(list,batchSize);
    }

    @Override
    @Transactional
    public boolean update(EAMWOType et) {
        if(!update(et,(Wrapper) et.getUpdateWrapper(true).eq("eamwotypeid",et.getEamwotypeid())))
            return false;
        CachedBeanCopier.copy(get(et.getEamwotypeid()),et);
        return true;
    }

    @Override
    public void updateBatch(List<EAMWOType> list) {
        updateBatchById(list,batchSize);
    }

    @Override
    @Transactional
    public boolean remove(String key) {
        boolean result=removeById(key);
        return result ;
    }

    @Override
    public void removeBatch(Collection<String> idList) {
        removeByIds(idList);
    }

    @Override
    @Transactional
    public EAMWOType get(String key) {
        EAMWOType et = getById(key);
        if(et==null){
            et=new EAMWOType();
            et.setEamwotypeid(key);
        }
        else{
        }
        return et;
    }

    @Override
    public EAMWOType getDraft(EAMWOType et) {
        return et;
    }

    @Override
    public boolean checkKey(EAMWOType et) {
        return (!ObjectUtils.isEmpty(et.getEamwotypeid()))&&(!Objects.isNull(this.getById(et.getEamwotypeid())));
    }
    @Override
    @Transactional
    public boolean save(EAMWOType et) {
        if(!saveOrUpdate(et))
            return false;
        return true;
    }

    @Override
    @Transactional
    public boolean saveOrUpdate(EAMWOType et) {
        if (null == et) {
            return false;
        } else {
            return checkKey(et) ? this.update(et) : this.create(et);
        }
    }

    @Override
    public boolean saveBatch(Collection<EAMWOType> list) {
        saveOrUpdateBatch(list,batchSize);
        return true;
    }

    @Override
    public void saveBatch(List<EAMWOType> list) {
        saveOrUpdateBatch(list,batchSize);
    }


	@Override
    public List<EAMWOType> selectByEamwostatemodelid(String eamwostatemodelid) {
        return baseMapper.selectByEamwostatemodelid(eamwostatemodelid);
    }

    @Override
    public void removeByEamwostatemodelid(String eamwostatemodelid) {
        this.remove(new QueryWrapper<EAMWOType>().eq("eamwostatemodelid",eamwostatemodelid));
    }


    /**
     * 查询集合 DEFAULT
     */
    @Override
    public Page<EAMWOType> searchDefault(EAMWOTypeSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<EAMWOType> pages=baseMapper.searchDefault(context.getPages(),context,context.getSelectCond());
        return new PageImpl<EAMWOType>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }







    @Override
    public List<JSONObject> select(String sql, Map param){
        return this.baseMapper.selectBySQL(sql,param);
    }

    @Override
    @Transactional
    public boolean execute(String sql , Map param){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql,param);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }

    @Override
    public List<EAMWOType> getEamwotypeByIds(List<String> ids) {
         return this.listByIds(ids);
    }

    @Override
    public List<EAMWOType> getEamwotypeByEntities(List<EAMWOType> entities) {
        List ids =new ArrayList();
        for(EAMWOType entity : entities){
            Serializable id=entity.getEamwotypeid();
            if(!ObjectUtils.isEmpty(id)){
                ids.add(id);
            }
        }
        if(ids.size()>0)
           return this.listByIds(ids);
        else
           return entities;
    }

}



