package cn.ibizlab.assetmanagement.core.workorder.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import cn.ibizlab.assetmanagement.core.workorder.domain.WOOrigin;
import cn.ibizlab.assetmanagement.core.workorder.filter.WOOriginSearchContext;
import cn.ibizlab.assetmanagement.core.workorder.service.IWOOriginService;

import cn.ibizlab.assetmanagement.util.helper.CachedBeanCopier;
import cn.ibizlab.assetmanagement.util.helper.DEFieldCacheMap;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cn.ibizlab.assetmanagement.core.workorder.mapper.WOOriginMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;
import org.springframework.util.StringUtils;

/**
 * 实体[工单来源] 服务对象接口实现
 */
@Slf4j
@Service("WOOriginServiceImpl")
public class WOOriginServiceImpl extends ServiceImpl<WOOriginMapper, WOOrigin> implements IWOOriginService {

    @Autowired
    @Lazy
    protected cn.ibizlab.assetmanagement.core.workorder.service.IEAMWorkOrderService eamworkorderService;

    protected int batchSize = 500;

    @Override
    @Transactional
    public boolean create(WOOrigin et) {
        if(!this.retBool(this.baseMapper.insert(et)))
            return false;
        CachedBeanCopier.copy(get(et.getWooriginid()),et);
        return true;
    }

    @Override
    public void createBatch(List<WOOrigin> list) {
        this.saveBatch(list,batchSize);
    }

    @Override
    @Transactional
    public boolean update(WOOrigin et) {
        if(!update(et,(Wrapper) et.getUpdateWrapper(true).eq("eamwooriginid",et.getWooriginid())))
            return false;
        CachedBeanCopier.copy(get(et.getWooriginid()),et);
        return true;
    }

    @Override
    public void updateBatch(List<WOOrigin> list) {
        updateBatchById(list,batchSize);
    }

    @Override
    @Transactional
    public boolean remove(String key) {
        boolean result=removeById(key);
        return result ;
    }

    @Override
    public void removeBatch(Collection<String> idList) {
        removeByIds(idList);
    }

    @Override
    @Transactional
    public WOOrigin get(String key) {
        WOOrigin et = getById(key);
        if(et==null){
            et=new WOOrigin();
            et.setWooriginid(key);
        }
        else{
        }
        return et;
    }

    @Override
    public WOOrigin getDraft(WOOrigin et) {
        return et;
    }

    @Override
    public boolean checkKey(WOOrigin et) {
        return (!ObjectUtils.isEmpty(et.getWooriginid()))&&(!Objects.isNull(this.getById(et.getWooriginid())));
    }
    @Override
    @Transactional
    public boolean save(WOOrigin et) {
        if(!saveOrUpdate(et))
            return false;
        return true;
    }

    @Override
    @Transactional
    public boolean saveOrUpdate(WOOrigin et) {
        if (null == et) {
            return false;
        } else {
            return checkKey(et) ? this.update(et) : this.create(et);
        }
    }

    @Override
    public boolean saveBatch(Collection<WOOrigin> list) {
        saveOrUpdateBatch(list,batchSize);
        return true;
    }

    @Override
    public void saveBatch(List<WOOrigin> list) {
        saveOrUpdateBatch(list,batchSize);
    }



    /**
     * 查询集合 DEFAULT
     */
    @Override
    public Page<WOOrigin> searchDefault(WOOriginSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<WOOrigin> pages=baseMapper.searchDefault(context.getPages(),context,context.getSelectCond());
        return new PageImpl<WOOrigin>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }







    @Override
    public List<JSONObject> select(String sql, Map param){
        return this.baseMapper.selectBySQL(sql,param);
    }

    @Override
    @Transactional
    public boolean execute(String sql , Map param){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql,param);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }

    @Override
    public List<WOOrigin> getWooriginByIds(List<String> ids) {
         return this.listByIds(ids);
    }

    @Override
    public List<WOOrigin> getWooriginByEntities(List<WOOrigin> entities) {
        List ids =new ArrayList();
        for(WOOrigin entity : entities){
            Serializable id=entity.getWooriginid();
            if(!ObjectUtils.isEmpty(id)){
                ids.add(id);
            }
        }
        if(ids.size()>0)
           return this.listByIds(ids);
        else
           return entities;
    }

}



